/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: CompareNumericDistributions.java,v 1.14 2006/04/27 17:10:19 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine;

import java.net.URL;
import java.text.NumberFormat;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.math.MathException;
import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;
import org.apache.commons.math.stat.inference.TestUtils;
import org.apache.log4j.Logger;

import prompt.input.filter.IFilterValue;
import prompt.statistic.KS;
import prompt.statistic.MannWhitney;
import prompt.util.ListTools;
import tschmidt.binningDialog.BinningDialog;
import tschmidt.binningDialog.BinningWizReturn;
import tschmidt.utils.Binning;
import tschmidt.utils.Binning2;
import tschmidt.utils.ValidLocation;


/**
 * Compares two numeric distributions tests significant differences. <br>
 * Result: <br>
 * 0: histogram values with absolute and relative frequency and the difference
 * between both sets <br>
 * 1: test result: Mann Whithney & KS und mean,median,std over the whole distribution <br>
 * 2..n: test results over all bins//  <br>
 * If a test failed or is not performed at all a null value is returned at
 * instead of an IEngineResult instance
 * 
 * @author schmidt
 *  
 */
public class CompareNumericDistributions {
	private final static long serialVersionUID = -6849794470754667700L;

	private Logger logger = Logger.getLogger(CompareNumericDistributions.class);

	// Stuff which is being returned via methods:
	/**
	 * histogram result string
	 */
	protected String results1;

	/**
	 * results of the test of the whole distribution
	 */
	protected IEngineResults testRes;

	/**
	 * array of test-results of all bins, elements which failed
	 */
	protected IEngineResults[] testBins;

	//changable from user via get and set
	/**
	 * Vom Benutzer veraenderbares Binning
	 */
	private Binning binning;

	private boolean testWholeDistribution = true;

	private boolean testEachBin = true;

	//internal only:
	/**
	 * Value lists after applying the value filter if given for setA
	 */
	protected List valuesA = null;

	/**
	 * Value lists after applying the value filter if given for setB
	 */
	protected List valuesB = null;

	/**
	 * Binning mit den gespeicherten werten die in die enstprechenden
	 * Intervalle(Bins) fallen vom setA bzw. B
	 */
	protected Binning2 binningA, binningB;

	protected String comparisonName, descA, descB;

	protected IFilterValue valFilter;

	public void init(String comparisonName, String descA, String descB, List valuesA, List valuesB, IFilterValue valFilter) throws Exception {
		this.valuesA = valuesA;
		this.valuesB = valuesB;
		this.comparisonName = comparisonName;
		this.descA = descA;
		this.descB = descB;
		this.valFilter = valFilter;
	}

	/**
	 * @param comparisonName
	 * @param descA
	 * @param descB
	 * @param valuesA
	 *             in Number format
	 * @param valuesB
	 *             in Number format
	 * @param valFilter
	 * @throws Exception
	 */
	public void init(String comparisonName, String descA, String descB, Object[] numbersA, Object[] numbersB, IFilterValue valFilter)
			throws Exception {
		this.valuesA = new LinkedList();
		this.valuesB = new LinkedList();
		NumberFormat nf = NumberFormat.getInstance(Locale.US);

		for (int i = 0; i < numbersA.length; i++) {
           
			String string = numbersA[i]+"";
			if ( string.equals("NaN")) continue;
			if (string.equals("null")) continue;
				Number l = nf.parse(string);
				valuesA.add(l);
			
		}
		for (int i = 0; i < numbersB.length; i++) {
			String string =  numbersB[i]+"";
			if ( string.equals("NaN")) continue;
			if (string.equals("null")) continue;
			
			Number l = nf.parse(string);
			valuesB.add(l);
		}
		this.comparisonName = comparisonName;
		this.descA = descA;
		this.descB = descB;
		this.valFilter = valFilter;
	}

	/**
	 * @param comparisonName
	 * @param descA
	 * @param descB
	 * @param valuesA
	 *            MUST BE STRINGS in Number format
	 * @param valuesB
	 *            MUST BE STRINGS in Number format
	 * @param valFilter
	 * @throws Exception
	 */
	public void init(String comparisonName, String descA, String descB, String[] numbersA, String[] numbersB, IFilterValue valFilter)
			throws Exception {
		this.valuesA = new LinkedList();
		this.valuesB = new LinkedList();
		NumberFormat nf = NumberFormat.getInstance(Locale.US);

		for (int i = 0; i < numbersA.length; i++) {
			String string = numbersA[i];
			Number l = nf.parse(string);
			valuesA.add(l);
		}
		for (int i = 0; i < numbersB.length; i++) {
			String string = numbersB[i];
			Number l = nf.parse(string);
			valuesB.add(l);
		}
		this.comparisonName = comparisonName;
		this.descA = descA;
		this.descB = descB;
		this.valFilter = valFilter;
	}

	/**
	 * wir haben die zwei Datenmengen in zwei Spalten und wollen diese beiden Spalten vergleichen
	 * @param comparisonName
	 * @param descA
	 * @param descB
	 * @param numbers String[][]
	 * @param colIndexA
	 * @param colIndexB
	 * @param valFilter
	 * @throws Exception
	 */
	public void init(String comparisonName, String descA, String descB, String[][] numbers, int colIndexA, int colIndexB, IFilterValue valFilter)
			throws Exception {
		this.valuesA = new LinkedList();
		this.valuesB = new LinkedList();
		NumberFormat nf = NumberFormat.getInstance(Locale.US);

		for (int i = 0; i < numbers.length; i++) {
			String[] string = numbers[i];
			Number l = nf.parse(string[colIndexA]);
			valuesA.add(l);
			Number l2 = nf.parse(string[colIndexB]);
			valuesB.add(l2);
		}

		this.comparisonName = comparisonName;
		this.descA = descA;
		this.descB = descB;
		this.valFilter = valFilter;
	}

	/**
	 * Generic constructor, must exist
	 */
	public CompareNumericDistributions() {
		super();
	}

	public String toString() {
		return results1;
	}

	protected String calcHist() {
		StringBuffer sb = new StringBuffer();

		Binning binning = getBinning();
		if (!binning.isInitAlready()) {
			String tmp[] = this.comparisonName.split("\\.");
			//BinningProperties retBin = BinningDialog.showDBDialog(null);
			BinningWizReturn retBin = BinningDialog.showBinningDialog(null,tmp[tmp.length-1]);
			if (retBin == null) {
				logger.debug("CANCELED HISTOGRAM CREATING");
				return "canceled";
			}
			
			
			//if (retBin.getType() == BinningProperties.TYPE_AUTO) {
			if (retBin.getType() == BinningWizReturn.TYPE_AUTO) {
				
				binning.autoInit(valuesA, valuesB);
				logger.debug("Try automatical binning since no binning was supplied");
							}
			//if (retBin.getType() == BinningProperties.TYPE_INTERVALS) {
			if (retBin.getType() == BinningWizReturn.TYPE_BIN) {
				
				//logger.debug("Check: "+tmp[tmp.length-1]);
				
				//binning.initNumberOfBins(retBin.getNumberofIntervals(), retBin.getMin(), retBin.getMax());
				//logger.debug("Binning in " + retBin.getNumberofIntervals() + " intervals ");
				binning = retBin.getBinning();
				setBinning(binning);
				//logger.debug("-----------------------------------------");
				//logger.debug("Check2IsInitAlready: "+binning.isInitAlready());
				//logger.debug("Check2: "+binning.toLabelString());
				//logger.debug("-----------------------------------------");
				
			}
			/*if (retBin.getType() == BinningProperties.TYPE_WIDTH) {
				logger.debug("BinningProperties:"+retBin.toString());
				binning.initBinWidth(retBin.getWidth(), retBin.getMin(), retBin.getMax());
			}*/
			//logger.debug(binning);
			//logger.debug(binning.toLabelString());

		}
		binningA = new Binning2(binning);
		binningB = new Binning2(binning);

		int[] MWabsA = null;
		int[] MWabsB = null;

		try {
			MWabsA = binningA.count(valuesA);
			MWabsB = binningB.count(valuesB);
		} catch (Exception e1) {
			logger.error("binning counting failed",e1);            
		}
		int arraysize = binning.getNumberOfBins();

		float[] MWrelA = binning.countRel(MWabsA);
		float[] MWrelB = binning.countRel(MWabsB);
		float[] MWdiff = new float[arraysize];

		for (int i = 0; i < arraysize; i++) {
			MWdiff[i] = MWrelA[i] - MWrelB[i];
		}

		//sb.append(binning.toLabelString() + "\n");

		int totalA = 0;
		int totalB = 0;
		
		sb.append("absA");
		for (int i = 0; i < arraysize; i++) {
			sb.append("\t" + MWabsA[i]);
			totalA = totalA+ MWabsA[i];
		}
		sb.append("\n");

		sb.append("relA");
		for (int i = 0; i < arraysize; i++) {
			sb.append("\t" + MWrelA[i]);
		}
		sb.append("\n");

		sb.append("absB");
		for (int i = 0; i < arraysize; i++) {
			sb.append("\t" + MWabsB[i]);
			totalB = totalB + MWabsB[i];
		}
		sb.append("\n");

		sb.append("relB");
		for (int i = 0; i < arraysize; i++) {
			sb.append("\t" + MWrelB[i]);
		}
		sb.append("\n");

		sb.append("relAMinusB");
		for (int i = 0; i < arraysize; i++) {
			sb.append("\t" + MWdiff[i]);
		}
		sb.append("\n");
		
		
		
		double total = (double) totalA + (double) totalB;
		if (total>0) {
			double frqA = totalA / total;
			double frqB = totalB /total;
			sb.append("Chi2_p_value");
			
			for (int i = 0; i < arraysize; i++) { //hier ueber alle Spalten
				long obsA = MWabsA[i];
				long obsB = MWabsB[i];
				double obsTotal = obsA+ obsB;
				double exp1 = frqA * obsTotal;
                double exp2 = frqB * obsTotal;
                //buf.append(exp1+"\t");
                //buf.append(exp2+"\t");
                double pval = Double.NaN;
                
                double[] expected = { exp1, exp2 };
                long[] observed = { obsA, obsB };
                try {
					pval = TestUtils.chiSquareTest(expected, observed);
				} catch (IllegalArgumentException e) {
					logger.debug("can not test as input is invalid ",e);
				} catch (MathException e) {
					logger.debug("Chi Square failed",e);
				}
				if (pval<0) pval = pval * -1.0d;
				
				sb.append("\t" + pval);
			}
			//sb.append("\n");
		
		} else {
			logger.warn("total number of observations is 0");
		}
		
		return sb.toString();

	}

	public IEngineResults cmp(List valA, List valB) throws Exception {
		StringBuffer sb = new StringBuffer("FIELD\tVALUE\n");

		double ksp = Double.NaN;
		try {
			KS ks = new KS();
			ks.init(valA, valB);
			ks.run();
			ksp = ks.getPValue();
		} catch (Exception e) {
			logger.debug("error testing whole distribution with KS ", e);
		}

		sb.append("KS_pvalue\t" + ksp + "\n");

		double mwp = Double.NaN;
		try {
			MannWhitney mw = new MannWhitney();
			mw.init(valA, valB, 0);
			mw.run();
			mwp = mw.getPValue();
		} catch (Exception e) {
			logger.debug("error testing whole distribution with MW ", e);
		}
		sb.append("MW_pvalue\t" + mwp + "\n");
		//logger.debug(mw.getResults());
		
		sb.append("setA_Description\t\"" + descA + "\"\n");
		sb.append("setB_Description\t\"" + descB + "\"\n");
		try {
			DescriptiveStatistics stats = DescriptiveStatistics.newInstance();
			//      Add the data from the array
			int i=0;
			for (Iterator iter = valA.iterator(); iter.hasNext();) {
				i++;
				Object element = (Object) iter.next();
				double va = Double.parseDouble(element + "");
				if (va == Double.NaN) {
					logger.warn(i + "-th entry is Not a Number (NaN)");
					continue;
				}
				if (va == Double.POSITIVE_INFINITY) {
					logger.warn(i + "-th entry is POSITIVE_INFINITY)");
					continue;
				}
				if (va == Double.NEGATIVE_INFINITY) {
					logger.warn(i + "-th entry is NEGATIVE_INFINITY)");
					continue;
				}
				stats.addValue(va);
			}
			//      Compute some statistics 
			double mean = stats.getMean();
			   double min = stats.getMin();
	            double max = stats.getMax();
			double std = stats.getStandardDeviation();
			double median = stats.getPercentile(50);
			sb.append("setA_mean\t" + mean + "\n");
			sb.append("setA_std\t" + std + "\n");
			sb.append("setA_median\t" + median + "\n");
			sb.append("setA_min\t" + min + "\n");
            sb.append("setA_max\t" + max + "\n");
		} catch (Exception e) {
			logger.debug("error calculation descriptive stats for set A", e);
		}

		try {
			DescriptiveStatistics stats = DescriptiveStatistics.newInstance();
			stats = DescriptiveStatistics.newInstance();
			//   Add the data from the array
			 int i=0;
			for (Iterator iter = valB.iterator(); iter.hasNext();) {
				i++;
				Object element = (Object) iter.next();
				double va = Double.parseDouble(element + "");
				
				if (va == Double.NaN) {
					logger.warn(i + "-th entry is Not a Number (NaN)");
					continue;
				}
				if (va == Double.POSITIVE_INFINITY) {
					logger.warn(i + "-th entry is POSITIVE_INFINITY)");
					continue;
				}
				if (va == Double.NEGATIVE_INFINITY) {
					logger.warn(i + "-th entry is NEGATIVE_INFINITY)");
					continue;
				}
				stats.addValue(va);

			}

			//   Compute some statistics 
			double mean = stats.getMean();
			   double min = stats.getMin();
	            double max = stats.getMax();
			double std = stats.getStandardDeviation();
			double median = stats.getPercentile(50);
			sb.append("setB_mean\t" + mean + "\n");
			sb.append("setB_std\t" + std + "\n");
			sb.append("setB_median\t" + median + "\n");
			sb.append("setB_min\t" + min + "\n");
	        sb.append("setB_max\t" + max + "\n");
		} catch (Exception e) {
			logger.debug("error calculation descriptive stats for set B", e);
		}

		
		IEngineResults res = new EngineResults();
		res.setResult(sb.toString(), "", "", true, comparisonName + ":statvalues");
		//res.setDesc1(comparisonName + " : " + descA + " vs. " + descB);
		res.setDesc1(descA + " vs. " + descB);
		res.setDesc2("Comparison whole distribution and tests");

		String desRes = null;
		URL url = getClass().getResource(this.getClass().getSimpleName() + "_whole.html");
		if (url == null) {
			desRes = "No description found :-(";
		} else {
		desRes = ValidLocation.readFromUrl(url);
		if ((desRes!=null) && (desRes.trim().length()>4)) {
			if (descA!=null) desRes = desRes.replaceAll("\\$SETA", descA);
			if (descB!=null) desRes = desRes.replaceAll("\\$SETB", descB);
		}
		}
		res.setResultDescription(desRes);
		return res;
	}

	public void run() throws Exception {

		ListTools.applyValFilterToList(valuesA, valFilter);
		ListTools.applyValFilterToList(valuesB, valFilter);

		//1. Histogrammwerte fuer den Plot
		results1 = calcHist();

		if (isTestWholeDistribution()) {
			testRes = cmp(valuesA, valuesB);
		}

		String labelstr = "";
		
		if ((isTestEachBin()) && (binning.isInitAlready())) {
			logger.debug("test each bin");
			//3. MannWhitney Tests ueber alle BINS. Fragestellung:
			// Unterscheiden sich die Vtlg. in den Bins
			testBins = new EngineResults[binningB.getNumberOfBins()];
			if (binningA.getNumberOfBins() != binningB.getNumberOfBins()) {
				throw new Exception("different number of bins for each set. This should never occur");
			}
			logger.debug("number of bins: " + testBins.length);

			StringBuffer label = new StringBuffer();
			for (int i = 0; i < binningA.getNumberOfBins(); i++) {
				MannWhitney testBin = new MannWhitney();
				IEngineResults testBinRes = null;
				float pvalue = 1;
				try {
					testBin.init(binningA.getValues(i), binningB.getValues(i), 0);
					testBin.run();
					testBinRes = testBin.getResults();
					pvalue = Float.parseFloat((String) testBin.getTestResults().get("$p.value"));

				} catch (Exception e) {
					logger.debug("TEST distribution of bin " + i + ": is null since there are too few samples");
					pvalue=Float.NaN;
                }
				label.append("\"");
				label.append(binningA.toLabelString(i));
				if (pvalue < 0.05f) {
					label.append(" [X]");
				}
                if (pvalue==Float.NaN) {
                    label.append(" [?]");
                }
				label.append("\"");
				if (i < binningA.getNumberOfBins() - 1) {
					label.append("\t");
				}

				if (testBinRes != null) {
					testBinRes = testBin.getResults();
					testBinRes.setDesc1(comparisonName + " : " + descA + " vs. " + descB);
					testBinRes.setDesc2("Test distribution of bin " + i + ": " + binning.toLabelString(i));
					String addon = "<br><br>Description line 1: "+testBinRes.getDesc1()+"<br>"+
					"Description line 2: "+testBinRes.getDesc2()+"<br></html>";
					
					testBinRes.setResultDescription(testBinRes.getResultDescription()+addon);
					logger.debug("TEST distribution of bin " + i + ": " + testBinRes);
					logger.debug(testBinRes.getMetaData());
				}
				testBins[i] = testBinRes;
			}
			labelstr = label.toString();

		} else {
			labelstr = binning.toLabelString();
		}
		logger.debug("labelstr '" + labelstr + "' ");
		results1 = labelstr + "\n" + results1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.jdrm.JobTemplate#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	/**
	 * @return Returns the binning object
	 */
	public Binning getBinning() {
		if (binning == null) {
			binning = new Binning();
		}
		return binning;
	}

	/**
	 * @param binning
	 *            The binning to set.
	 */
	public void setBinning(Binning binning) {
		this.binning = binning;
	}

	public IEngineResults[] getResultsAll() throws Exception {
		int binTestRes = 0;
		if (testBins != null) {
			binTestRes = testBins.length;
		}
		IEngineResults[] results = new EngineResults[2 + binTestRes];
		results[0] = new EngineResults();

		logger.debug("this.toString():'" + this.toString() + "'");
		//logger.debug()
		if ((results1!=null) && (results1.indexOf("canceled") == -1)) {
			logger.debug("FILL RESULT HISTOGRAM");
			results[0].setResult(this.toString(), descA, descB, true, comparisonName);
			String desRes = null;
			URL url = getClass().getResource(this.getClass().getSimpleName() + "_hist.html");
			if (url == null) {
				desRes = "No description found :-(";
			}
			else { 
				desRes = ValidLocation.readFromUrl(url);
				if ((desRes!=null) && (desRes.trim().length()>4)) {
					if (descA!=null) desRes = desRes.replaceAll("\\$SETA", descA);
					if (descB!=null) desRes = desRes.replaceAll("\\$SETB", descB);
				}
			}
			results[0].setResultDescription(desRes);
		}

		if (isTestWholeDistribution()) {
			if (testRes != null)
				results[1] = testRes;
		}
		if (isTestEachBin()) {
			if (testBins != null) {
				for (int i = 0; i < testBins.length; i++) {
					IEngineResults currentBinRes = testBins[i];
					results[2 + i] = currentBinRes;
				}
			}
		}
		return results;
	}

	/**
	 * @return whether a test of distribution differences of all bins should be
	 *         performed
	 */
	public boolean isTestEachBin() {
		return testEachBin;
	}

	/**
	 * @param boolean,
	 *            default is true
	 */
	public void setTestEachBin(boolean testEachBin) {
		this.testEachBin = testEachBin;
	}

	/**
	 * @return returns whether a test of distribution differences of the whole
	 *         distribution should be performed
	 */
	public boolean isTestWholeDistribution() {
		return testWholeDistribution;
	}

	/**
	 * @param boolean,
	 *            default is true
	 */
	public void setTestWholeDistribution(boolean testWholeDistribution) {
		this.testWholeDistribution = testWholeDistribution;
	}
}
