/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: GeneMerge.java,v 1.5 2006/03/19 11:58:12 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genemerge;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.log4j.Logger;

import prompt.engine.Engine;
import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.ui.geneMergeDialog.GeneMergeDialog;
import prompt.ui.geneMergeDialog.GeneMergeProperties;
import tschmidt.iter.FileLineIter;
import tschmidt.utils.SimpleHashDouble;
import tschmidt.utils.SimpleHashInteger;
import tschmidt.utils.ValidLocation;



/**
 * 	
	 This program returns P-Values for functional enrichment in a cluster
	 of study genes using the hypergeometric distribution, as well 
	 functional descriptions for each gene. 

	 Input on command line is as follows:

	 file1 = a gene-association ID file
	 file2 = a human readable description file for gene-association IDs 
	 file3 = a population set of genes (the pool from which the study set
	 is drawn, often a genome or all genes on a microarray
	 file4 = a study set of genes (for example, genes found up/down regulated)
	 file5 = an output filename

	 n = # population genes
	 k = # study set genes
	 p = # of genes with a particular GMRG term divided by total genes (n)
	 r = # of genes in study set with a particular term
	
 * 
 * @author tschmidt (JAVA port + Extensions)
 *
 */
public class GeneMerge extends Engine {

	
	private static final long serialVersionUID = 1L;

	// Set input files and arguments
	/**
	 * format: genename \t GMRG id
	 */
	String gene_association_file;

	/** 
	 * format: GMRG id \t English description
	 */
	String description_file;

	/**
	 * genenames of "population set"
	 */
	String population_genes_file;

	/**
	 * genenames of "study set"
	 */
	String study_genes_file;
	
	
	public String label_association;
	public String label_description;
	public String label_pop;
	public String label_study;
	public String label_resulttype = this.getClass().getSimpleName();
	
	protected String writeTmpFile(String data,String prefix) throws IOException {
		
	File file = File.createTempFile("GM_"+prefix, ".tmp");
		BufferedWriter out = new BufferedWriter(
            new FileWriter(file));

    out.write(data);
    out.flush();
    out.close();
    return file.getAbsolutePath();
	}
	
	 /**
	 * @param gene_association_file
	 * @param description_file
	 * @param population_genes_file
	 * @param study_genes_file
	 */
	public void init( 
			 String	gene_association_file,
			 String description_file,
			 String population_genes_file,
			 String study_genes_file)
			 throws Exception {
		setGene_association_file(gene_association_file);
		setDescription_file(description_file);
		setPopulation_genes_file(population_genes_file);
		setStudy_genes_file(study_genes_file);	 
	}
		
	
	/**
	 * Init with data instead of filenames
	 * @param gene_associations
	 * @param descriptions
	 * @param population_genes
	 * @param study_genes
	 * @throws Exception
	 */
	public void initInMem( String gene_associations,
			 String descriptions,
			 String population_genes,
			 String study_genes) throws Exception
	{
		gene_association_file=writeTmpFile(gene_associations,"anno");
		if (descriptions!=null) description_file=writeTmpFile(descriptions,"desc");
		population_genes_file=writeTmpFile(population_genes,"pop");
		study_genes_file=writeTmpFile(study_genes,"study");
	} 
	
	
	
	protected HashMap<String, String> getHashedData(String filename) {
		if (filename==null) return null;
        HashMap<String, String> hash = new HashMap<String, String>();
		//if (isReadOk(filename)) {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(filename));
			String thisLine;
			while ((thisLine = br.readLine()) != null) {
				thisLine = thisLine.trim();
				String[] split = thisLine.split("\\t", 2);
				hash.put(split[0], split[1]);
			}

		} catch (Exception e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
		} finally {
			try {
				br.close();
			} catch (IOException e1) {
                Logger logger = Logger.getLogger(this.getClass());
                logger.error(e1);
			}
		}
		//}
		return hash;
	}
	
	/**
	 * reads from <code>gene_association_file</code>
	 * @return Hash: genename --> Annotations/Associated ids e.g. GOs, FunCat, Structures like Folds IDS
	 * Annotationids mit Strichpunkt trennen
	 * (diese Annotation IDs koennen dann in dem Description file beschrieben sein) 
	 */
	private HashMap<String, String> getAnnotationData() {
		HashMap<String, String> res =  getHashedData(gene_association_file);
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Done parsing '" + gene_association_file + "'");
		return res;
	}
	
	

	private HashMap<String, String> getAnnotationDescription() {
		HashMap<String, String> res =getHashedData(description_file);
		Logger logger = Logger.getLogger(this.getClass());
		logger.info("Done parsing '" + description_file + "'");
		return res;
	}
	
	
	
	class AnnotationCompared {
		String annoTerm;
		double popFreq;        
		String popFrac;        
		String studyFrac;
		double studyFreq;
		double diffStudyMinusPop;
		double raweScore;  
		double eScore;
		String description;
		LinkedList contributinGenes;
		
		@Override
		public String toString() {
			StringBuilder buf = new StringBuilder();
			buf.append(annoTerm); buf.append("\t");
			buf.append(popFreq); buf.append("\t");
			buf.append(popFrac); buf.append("\t");
			buf.append(studyFrac); buf.append("\t");
			buf.append(studyFreq); buf.append("\t");
			buf.append(diffStudyMinusPop); buf.append("\t");
			buf.append(raweScore); buf.append("\t");
			buf.append(eScore); buf.append("\t");
			buf.append(description); buf.append("\t");
			buf.append(contributinGenes); //buf.append("\n");
			return buf.toString();
		}
		
	}
	
	@Override
	public void run() throws Exception {
		Logger logger = Logger.getLogger(this.getClass());
		logger.info(this.getClass().getName());
		
		boolean askUser = false;
		if (this.getGene_association_file()==null) askUser=true;
		if (this.getPopulation_genes_file()==null) askUser=true;
		if (this.getStudy_genes_file()==null) askUser=true;
		//if (this.getDescription_file()==null) askUser=true;
		
		if ( askUser ) {
		 
			GeneMergeProperties settings = GeneMergeDialog.showGeneMergeDialog(null);
			if (settings==null) return; //user canceled
			
			setGene_association_file(settings.getGeneAssFile().getAbsolutePath());
			setPopulation_genes_file(settings.getPopulationFile().getAbsolutePath());
			setStudy_genes_file(settings.getStudyFile().getAbsolutePath());
			setDescription_file(settings.getDescFile().getAbsolutePath());
		}
		
		// Reads the gene annotations from the association file
		HashMap<String, String> GMRGgenomehash = getAnnotationData();
		
		// Reads the descriptions of the annotations ids that were found in the association file
		HashMap<String, String> annoDescHash = getAnnotationDescription();
	

		//Count the annotations in the POPULATION File // ll 165-
		int total_no_detected_genes = 0; //GLOBAL STAT
		SimpleHashInteger GMRGcount_hash = new SimpleHashInteger();

		FileLineIter fiter = new FileLineIter(population_genes_file); 
		while (fiter.hasNext()) {
			String gene = (String) fiter.next();
			gene = gene.trim();
			if (gene.length() > 0) {
				total_no_detected_genes++;
				String bigline = GMRGgenomehash.get(gene);
				if (bigline != null) {
					String[] indivGMRGs = bigline.split(";"); //Annotation ids are seperated with ;

					for (String indiv : indivGMRGs) {
						indiv = indiv.trim();
						GMRGcount_hash.inc(indiv);
					}
				}
			}
		}
		fiter.close();
		logger.info("Number of population genes: " + total_no_detected_genes);

		SimpleHashDouble GMRGfreq_hash = new SimpleHashDouble();
		Iterator i = GMRGcount_hash.keySet().iterator();
		while (i.hasNext()) {
			String currentKey = (String) i.next();
			double currentValue = GMRGcount_hash.get(currentKey);
			double newValue = currentValue / ((double) total_no_detected_genes);
			GMRGfreq_hash.put(currentKey, newValue);
			logger.info(currentKey + " = " + GMRGcount_hash.get(currentKey) + "\t freq = " + newValue);
		}

		//Determine GRMGID freqs in up/down regulated genes IN STUDY FILE ll. 252-
		int total_no_updown_genes = 0; //GLOBAL STAT
		int GenesNoInfo = 0; //GLOBAL STAT
		LinkedList<String> genes_not_found_in_ontology = new LinkedList<String>(); //GLOBAL STAT
		SimpleHashInteger updownGMRGcount_hash= new SimpleHashInteger();
		HashMap<String,LinkedList<String>> annoUsedByGenes = new HashMap<String,LinkedList<String>>();
		
		FileLineIter fstudy = new FileLineIter(study_genes_file);
		while (fstudy.hasNext()) {
			String gene = (String) fstudy.next();
			gene = gene.trim();
			if (gene.length() > 0) {
				total_no_updown_genes++;
				String fullline = GMRGgenomehash.get(gene);
				if (fullline!=null) {
					String[] indivGMRGs = fullline.split(";"); //Annotation ids are seperated with ;
					for (String indiv : indivGMRGs) {
						indiv = indiv.trim();
						updownGMRGcount_hash.inc(indiv);
						LinkedList<String> list = annoUsedByGenes.get(indiv);
						if (list==null) list=new LinkedList<String>();
						list.add(gene);
						annoUsedByGenes.put(indiv,list);
					}
					
				} else {
					genes_not_found_in_ontology.add(gene);
					GenesNoInfo++;
				}
				
			} // gene length >0
		}
		fstudy.close();
		logger.info("Number of study set genes: "+total_no_updown_genes);
		
		int no_of_up_down_GMRG_terms = 0;//GLOBAL STAT
		int BCr = 0;//GLOBAL STAT
		Iterator i2 = updownGMRGcount_hash.keySet().iterator();
		while (i2.hasNext()) {
			String currentKey = (String) i2.next();
			int counts = updownGMRGcount_hash.get(currentKey);
			logger.info(currentKey + " = " + counts);
			
			// count the number of unique GMRG terms in up/down genes
		    no_of_up_down_GMRG_terms++;
		    /* count the number of GMRG terms that are not represented only once on the array
		    (these will be in frequency 1/total detected genes)
		     we use this figure in Bonferroni correction later */
		    double minFreq = 1d/(double)total_no_detected_genes;
		    if (GMRGfreq_hash.get(currentKey) > minFreq) BCr++;
		}
		logger.info("\n\nNumber of study set GMRGterms is = "+no_of_up_down_GMRG_terms);
		logger.info("BCr = "+BCr);
		
		StringBuilder buf2 = new StringBuilder();
		
		

		
		buf2.append("annoTerm\tpopFreq\tpopFrac\tstudyFrac\tstudyFreq\tdiffStudyMinusPop\traweScore\teScore\tdescription\tcontributinGenes\n");
		
		LinkedList<AnnotationCompared> results = new LinkedList<AnnotationCompared>();
		Iterator i3 = updownGMRGcount_hash.keySet().iterator();
		
		while (i3.hasNext()) {
			String key = (String) i3.next();
			int value = updownGMRGcount_hash.get(key); //+r
			double freq = GMRGfreq_hash.get(key); //p
			AnnotationCompared res = new AnnotationCompared();
			res.annoTerm=key;
			res.popFreq=freq;
			int count = 0;
			if (GMRGcount_hash.containsKey(key)) count =GMRGcount_hash.get(key); 
			res.popFrac=count+"/"+total_no_detected_genes;
			res.studyFrac=value+"/"+total_no_updown_genes;
			res.studyFreq = ((double) value) / ((double) total_no_updown_genes);
			res.diffStudyMinusPop = res.studyFreq - res.popFreq;
			//CALC p-values
			//n=total_no_detected_genes
			//k=total_no_updown_genes
			//r=value
			double pvalue = Double.NaN;
			double pvalue_corrected= Double.NaN;
			if (value >1) {
				pvalue= hypergeometric(total_no_detected_genes, freq, total_no_updown_genes, value);
				pvalue_corrected= (pvalue* ((double) BCr));
				if (pvalue_corrected>=1d) pvalue_corrected=1d;
				
			} 
			logger.info("pvalue is: "+pvalue);
			logger.info("Bonferroni corrected pvalue is: "+pvalue_corrected);
			res.raweScore=pvalue;
			res.eScore=pvalue_corrected;
			
			//look up description
            if (annoDescHash!=null) 
			res.description=annoDescHash.get(key);
            else res.description="NA";
			res.contributinGenes= annoUsedByGenes.get(key);
			buf2.append(res+"\n");
			results.add(res);
			//buf2.append(key+"\t"+total_no_detected_genes+"\t"+freq+"\t"+total_no_updown_genes+"\t"+value+"\n");
		}
		res1 = buf2.toString();
		
		
		StringBuilder buf3 = new StringBuilder();
		buf3.append("VAR\tVALUE\n");
		buf3.append("Total number of genes\t"); buf3.append(total_no_detected_genes); buf3.append("\n");
		buf3.append("Total number of Study genes/proteins\t"); buf3.append(total_no_updown_genes); buf3.append("\n");

		buf3.append("Total number of Study gene/protein Annotation terms (pop non-singletons)\t"); buf3.append(no_of_up_down_GMRG_terms+" ("+BCr+")"); buf3.append("\n");
		
	
		buf3.append("Study-genes with Annotation information\t"); buf3.append(total_no_updown_genes-GenesNoInfo); buf3.append("\n"); 
		buf3.append("Study-Genes with no Annotation information\t"); buf3.append(GenesNoInfo); buf3.append("\n");
		buf3.append("These are:\n");
		for (String gene : genes_not_found_in_ontology) {
			buf3.append(gene); buf3.append("\n");
		}
		
		res2 = buf3.toString();
		
		//key_n_p_k_r= buf2.toString();
		//logger.info(key_n_p_k_r);
		
		//PART III> Get p_values for each annotation ID in the study set

		
	}

	
	
	
	
	/*public String getMetaData() {
		return "<table><tr>" 
				+"<td>gene_association_file</td><td>"+getGene_association_file()+"</td></tr>"
				+"<td>description_file</td><td>"+getDescription_file()+"</td></tr>"
				+"<td>population_genes_file</td><td>"+getPopulation_genes_file()+"</td></tr>"
				+"<td>study_genes_file</td><td>"+getStudy_genes_file()+"</td></tr>"
				+"</table>";
	} */

	public String getDescription_file() {
		return description_file;
	}

	public void setDescription_file(String description_file) {
		this.description_file = description_file;
	}

	public String getGene_association_file() {
		return gene_association_file;
	}

	public void setGene_association_file(String gene_association_file) {
		this.gene_association_file = gene_association_file;
	}

	public String getPopulation_genes_file() {
		return population_genes_file;
	}

	public void setPopulation_genes_file(String population_genes_file) {
		this.population_genes_file = population_genes_file;
	}

	public String getStudy_genes_file() {
		return study_genes_file;
	}

	public void setStudy_genes_file(String study_genes_file) {
		this.study_genes_file = study_genes_file;
	}

	
	private String res1;
	private String res2;
	
	
	
	@Override
	public String toString() {
		return res1;
	}

	
	
	protected String addSetDescriptions() {
		String desRes = "";
		desRes = desRes + "Associations: ";
		if (label_association==null) desRes = desRes + this.getGene_association_file(); else desRes = desRes + label_association;
		desRes = desRes + "<br>";

		desRes = desRes + "Description: ";
		if (label_description==null) desRes = desRes + this.getDescription_file(); else desRes = desRes + label_description;
		desRes = desRes + "<br>";
		
		desRes = desRes + "Population: ";
		if (label_pop==null) desRes = desRes + this.getPopulation_genes_file(); else desRes = desRes + label_pop;
		desRes = desRes + "<br>";
		
		desRes = desRes + "Study: ";
		if (label_study==null) desRes = desRes + this.getStudy_genes_file(); else desRes = desRes + label_study;
		desRes = desRes + "<br></html>";
		return desRes;
	}
	
	@Override
	public IEngineResults getResults() throws Exception {
		IEngineResults res= new EngineResults();
		  res.setResult(
				res1,
				desc1,
				desc2,
				rownames,
				label_resulttype);
			URL url = getClass().getResource(this.getClass().getName() + "Result1.html");
			String desRes = "No description found :-(";
			if (url != null) desRes = ValidLocation.readFromUrl(url);
			desRes= addSetDescriptions();
			
		  res.setResultDescription(desRes);
		  return res;
	}

	@Override
	public IEngineResults[] getResultsAll() throws Exception {
		IEngineResults[] results = new EngineResults[2];
		  results[0]= getResults();
		  results[1]= new EngineResults();
		  results[1].setResult(
				res2,
				desc1,
				desc2,
				rownames,
				label_resulttype); 
			URL url = getClass().getResource(this.getClass().getName() + "Result2.html");
			String desRes = "No description found :-(";
			if (url != null) desRes = ValidLocation.readFromUrl(url);
			desRes= addSetDescriptions();
			results[1].setResultDescription(desRes);
		return results;
	}

	/**
	 * key =each annotation-identifier that is associated with genes of the study
	 * n= total number of detected genes (population size)
	 * p= freq of the key in the population
	 * k= number of genes in the study (sample size)
	 * r= number of occurences of key in the study set
	 * @return
	 */
	/*public String getKey_n_p_k_r() {
		return key_n_p_k_r;
	}*/


	
	/**
	 Hypergeometric tail probability<br>
	 <br>
	 Returns one-tailed probability based on the hypergeometric distribution:<br>
	 the probability of witnessing r successes in a sample of k items from
	 a pool of size n, with r having prob (proportion) p, sampling without
	 replacement. Uses natural log n choose k and factorial subroutines.
	 */
	private double hypergeometric(int n, double p, int k, int r) {
		//see also  HypergeometricDistribution (Apache Math commons)
		double q;
		int np;
		int nq;
		int top;

		q = (1 - p);
		np = (int) Math.round((double) n * p);
		nq = (int) Math.round((double) n * q);

		double log_n_choose_k = lNchooseK(n, k);

		top = k;
		if (np < k) {
			top = np;
		}

		double foo = lNchooseK(np, top) + lNchooseK(nq, k - top);
		double sum = 0;

		for (int i = top; i >= r; i--) {
			sum = sum + Math.exp(foo - log_n_choose_k);

			if (i > r) {
				foo = foo + Math.log((double) i / ((double) (np - i + 1)));
				foo = foo + Math.log((double) (nq - k + i) / ((double) (k - i + 1)));
			}
		}
		return sum;
	}

	/**
	 * ln factorial
	 * @param number
	 * @return double
	 */
	private double lFactorial(double number) {
		double returnValue = 0;
		for (int i = 2; i <= number; i++) {
			returnValue = returnValue + Math.log(i);
		}
		return returnValue;
	}

	/**
	 * ln N choose K subroutine
	 * @param n
	 * @param k
	 * @return double
	 */
	private double lNchooseK(int n, int k) {
		double answer = 0;

		if (k > (n - k)) {
			k = (n - k);
			;
		}
		for (int i = n; i > (n - k); i--) {
			answer = answer + Math.log(i);
		}
		answer = answer - lFactorial(k);
		return answer;
	}

	public void setDesc1(String s1) {
	 this.desc1=s1;	
	}
	
	public void setDesc2(String s2) {
		 this.desc2=s2;	
		}
}
