/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Weka2Xml_InMemory.java,v 1.1 2006/03/19 20:35:54 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import prompt.engine.Engine;
import prompt.ui.SaveUI;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffLoader;


public class Weka2Xml_InMemory extends Engine {

    private static final long serialVersionUID = 1L;

    String srcFileName = null;
    String destFileNamePrefix = null;
    String[] resultFiles = null;
    
    boolean worked = false;
    
    
    @Override
    public void run() throws Exception {
        worked = false;
    	Logger logger = Logger.getLogger(Weka2Xml_InMemory.class.getName());
        if (srcFileName==null) {
          srcFileName = SaveUI.load("Load a WEKA arff file", ".arff");
        }
        if (srcFileName == null) {
            String errmsg = "Converting canceled (no src file choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }

        File srcFile = new File(srcFileName);

        try {
            ArffLoader atf = new ArffLoader();
            atf.setSource(srcFile);

            /*1. get class attributes, ask filename for each class attrib
             if class attrib is numeric -> abort
             
             */
            Instances struc = atf.getStructure();
            logger.debug(struc);
            int numAttr = struc.numAttributes();
            if (numAttr <= 1) {
                String errmsg = "Not enough attributes in the WEKA file. <br>You need a class attribute and at least one feature attribute.";
                logger.fatal(errmsg);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
                return;
            }
            int clIndex = struc.classIndex();
            if (clIndex == -1) {
                //use weka default the last one
                clIndex = numAttr - 1; // erstes Attribut hat den Index 0
                struc.setClassIndex(clIndex);
                logger.debug("set class attribute to index nr " + clIndex);

            }
            if (struc.checkForStringAttributes()) {
                logger.warn("Attention STRING attributes found. They will not converted");
            }
            Attribute clAttr = struc.classAttribute();
            if (!clAttr.isNominal()) {
                String errmsg = "The last class attribute '" + clAttr.name() + "' must be of the type nominal";
                logger.fatal(errmsg);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
                return;
            }
            int clNumValues = clAttr.numValues();
            if (clNumValues == 0) {
                String errmsg = "The last class attribute '" + clAttr.name() + "' needs at least one value";
                logger.fatal(errmsg);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
                return;
            }

            Element[][] vec = new Element[clNumValues][numAttr - 1]; //ohne class attribut 
            for (int i = 0; i < clNumValues; i++) {
                for (int j = 0; j < numAttr - 1; j++) {
                    Element el = new Element("property");
                    Attribute wekaAttr = struc.attribute(j);
                    el.setAttribute("id", wekaAttr.name());
                    if (wekaAttr.isNumeric())
                        el.setAttribute("type", "numeric");
                    if (wekaAttr.isNominal())
                        el.setAttribute("type", "symbolic");
                    if (wekaAttr.isString())
                        el = null; // ignore  
                    if (wekaAttr.isDate())
                        el = null; //ignore
                    vec[i][j] = el;
                }
            }
            resultFiles = new String[clNumValues];
            int[] protCount = new int[clNumValues];

            Instance temp;
            int i = 0;
            do {
                temp = atf.getNextInstance();
                if (temp != null) {
                    i++;
                    //logger.debug(temp);
                    temp.setDataset(struc);
                    //logger.debug(temp.classIndex());
                    if (temp.classIsMissing()) {
                        logger.warn("Class attribute is missing at instance number " + i);
                        continue;
                    }
                    int currentIndex = (int) (temp.classValue());
                    String out = resultFiles[currentIndex];
                    if (out == null) {
                        
                    	String destFileName = null;
                    	if (destFileNamePrefix==null) {
                    		//ask user about the output file
                    		destFileName= SaveUI.save("Save data of '" + clAttr.name() + "'='" + clAttr.value(currentIndex) + "'", ".xml");
                    	} else {
                    		destFileName=destFileNamePrefix+"_"+clAttr.name() + "_" + clAttr.value(currentIndex);
                    	}
                        
                        if (destFileName == null) {
                            String errmsg = "Converting canceled (destination file not choosen)";
                            logger.fatal(errmsg);
                            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
                            return;
                        }

                        resultFiles[currentIndex] = destFileName;
                        logger.debug("file " + currentIndex + " is '" + destFileName + "'");
                    }
                    protCount[currentIndex]++;
                    int currentId = protCount[currentIndex];

                    //gehe ueber alle attribute -> properties und baue ueber einen protein record dazu
                    for (int za2 = 0; za2 < numAttr - 1; za2++) {
                        Attribute atr = temp.attribute(za2);
                        double d = temp.value(za2);
                        String value = d + ""; // if nomial, then replace double with string from struc
                        if (atr.isNominal()) value= atr.value((int)d);
                        Element property = vec[currentIndex][za2]; // von der Classe und dem aktuellen Attribut
                        if (property != null) { //ignore date/string attributes
                            Element item = new Element("input");
                            item.setAttribute("id", currentId + "");
                            item.setAttribute("value", value);
                            property.addContent(item);
                        }
                    }

                } // instance not null
            } while (temp != null);

            //Ausgabe
            for (int j = 0; j < vec.length; j++) {
                Element[] props = vec[j];
                //1. erzeuge XML Documents
                Document doc = new Document();
                Element xmlRoot = new Element("dataset");
                xmlRoot.setAttribute("label", struc.relationName() + " " + clAttr.name() + "=" + clAttr.value(j));
                doc.addContent(xmlRoot);
                //attach properties
                for (Element element : props) {
                    if (element != null)
                        xmlRoot.addContent(element);
                }

                XMLOutputter outputter = new XMLOutputter();
                Format fmt = outputter.getFormat();
                fmt.setIndent("  ");
                fmt.setLineSeparator("\n");
                outputter.setFormat(fmt);
                FileOutputStream fs = null;
                try {
                    logger.debug("save Generic XML file '" + resultFiles[j] + "'");
                    fs = new FileOutputStream(resultFiles[j]);
                } catch (FileNotFoundException e) {
                    String errmsg = "Output filename is invalid";
                    logger.fatal(errmsg);
                    JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);
                    return;
                }
                try {
                    outputter.output(doc, fs);
                } catch (IOException e1) {
                    String errmsg = "I/O Error saving Generic XML file '" + resultFiles[j] + "'";
                    logger.fatal(errmsg);
                    JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e1);
                    return;
                }
                if (fs!=null) fs.close();
            }

        } catch (Exception ex) {
            String errmsg = "Error converting the WEKA arff file '" + srcFileName + "'";
            logger.error(errmsg, ex);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, ex);
            return;
        }
        worked=true;
        String errmsg = "Finished successfully";
        logger.info(errmsg);
        JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
        
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return null;
    }

	public String[] getResultFiles() {
		return resultFiles;
	}

	public String getDestFileNamePrefix() {
		return destFileNamePrefix;
	}

	public void setDestFileNamePrefix(String destFileNamePrefix) {
		this.destFileNamePrefix = destFileNamePrefix;
	}

	public String getSrcFileName() {
		return srcFileName;
	}

	public void setSrcFileName(String srcFileName) {
		this.srcFileName = srcFileName;
	}
	/**
	 * @return true, if the last run() finished successfully
	 */
	public boolean hasWorked() {
		return worked;
	}


}
