/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ICath.java,v 1.2 2006/01/05 02:19:13 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface ICath  {

	/**
	 * Im Fall von PEDANT2: ist die Tabelle cath ueberhaupt da 
	 * und ist was drin, dann true
	 * @return boolean
	 */
	public boolean isCathAvailable();
	
	
	/**
	 * Returns for each protein all cath-class numbers that are better
	 * than the choosen threshold<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 3;2;3;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * CATH class Ids
	 */
	public HashMap getCathClassIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the CATH
	 * class numbers that are within the pedant dataset<br>
	 * <b>Example:</b><br>
	 * classnr -> plain description<br>
	 * 3 -> Alpha Beta<br>
	 * 2 -> Mainly Beta<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getCathClassDesc();
	
	
	/**
	 * Returns for each protein all cath-class+arc numbers that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 3.90;2.20;3.40;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * CATH class Ids & architecture numbers (Format class.arc)
	 */
	public HashMap getCathArcIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the CATH
	 * class numbers & Architecture Numbers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * classnr.ArcNr -> plain description<br>
	 * 3.40 -> 3-Layer(aba) Sandwich<br>
	 * 2.40 -> Barrel<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getCathArcDesc();
	
	
	
	/**
	 * Returns for each protein all cath-class+arc+topol numbers that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 3.90.750;2.120.10;2.40.128;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * CATH class Ids & architecture numbers (Format class.arc.topol)
	 */
	public HashMap getCathTopolIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the CATH
	 * class numbers & Architecture Numbers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * classnr.Arc.Topol -> plain description<br>
	 * 3.90.750 -> Homoserine Dehydrogenase; Chain A, domain 2h<br>
	 * 2.120.10 -> Neuraminidase<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getCathTopolDesc();
	
	
	/**
	 * Returns for each protein all the full CATH (all 4 levels) numbers that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 3.90.750.10;3.40.50.720;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * CATH class Ids & architecture numbers (Format class.arc.topol.homol)
	 */
	public HashMap getCathHomolIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the CATH
	 * class numbers & Architecture Numbers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * classnr.Arc.Topol.Homol -> plain description<br>
	 * 3.90.750.10 -> Homoserine Dehydrogenase; Chain A, domain 2<br>
	 * 3.40.50.720 -> NAD(P)-binding Rossmann-like Domain<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getCathHomolDesc();
	
	
}
