/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IScop.java,v 1.2 2006/01/05 02:19:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IScop {

	/**
	 * Sind ueberhaupt SCOP-Klassifikationen da
	 * (im Fall von PEDANT2 muessen mehrere Tabellen ueberprueft werden) 
	 * dann true
	 * @return boolean
	 */
	public boolean isScopAvailable();
	
	
	/**
	 * Returns for each protein all SCOP-<b>class</b> letters that are better
	 * than the choosen threshold<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> d;c;a;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * SCOP class letters
	 */
	public HashMap getScopClassIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the SCOP
	 * <b>class</b> letters that are within the pedant dataset<br>
	 * <b>Example:</b><br>
	 * classnr -> plain class description<br>
	 * d -> Alpha and beta proteins (a+b)<br>   
	 * c -> Alpha and beta proteins (a/b)<br>  
	 * @param eValThreshold
	 * @return HashMap
	 */ 
	public HashMap getScopClassDesc();
	
	
	/**
	 * Returns for each protein all SCOP-<b>class+fold</b> numbers that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> d.81;c.2;c.73;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * SCOP class letters & Fold numbers (Format class.fold)
	 */
	public HashMap getScopFoldIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the SCOP
	 * <b>Folds</b> that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * class_letter.Fold -> plain fold description<br>
	 * d.81 -> Glyceraldehyde-3-phosphate dehydrogenase-like, C-terminal domain<br>
	 * c.2 -> NAD(P)-binding Rossmann-fold domains<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getScopFoldDesc();
	

	
	/**
	 * Returns for each protein all SCOP-class+fold+superfamily numbers that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> d.81.1;c.2.1;c.73.1....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * SCOP Class+ Fold+ Superfamily (Format class.fold.superfamily)
	 */
	public HashMap getScopSuperfamilyIds(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the SCOP
	 * <b>superfamilies</b> that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * class_letter.fold.superfamily -> plain superfamily description<br>
	 * d.81.1 -> Glyceraldehyde-3-phosphate dehydrogenase-like, C-terminal domain<br>
	 * c.2.1 -> NAD(P)-binding Rossmann-fold domains<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getScopSuperfamilyDesc();
	

	/**
	 * Returns for each protein all full SCOP numbers that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> d.81.1.2;c.2.1.3;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * SCOP Class+ Fold+ Superfamily + family (Format class.fold.superfamily.family)
	 */
	public HashMap getScopFamilyIds(String eValThreshold);
	

	
	/**
	 * Returns plain human-readable descriptions of the SCOP
	 * <b>families</b> that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * class_letter.fold.superfamily.family -> plain family description<br>
	 * d.81.1.2 -> Homoserine dehydrogenase-like<br>
	 * c.2.1.3 -> Glyceraldehyde-3-phosphate dehydrogenase-like, N-terminal domain<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getScopFamilyDesc();
	
	
}
