/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: FlatFile.java,v 1.13 2006/04/06 13:09:51 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import gr.zeus.ui.JMessage;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.seq.io.SeqIOTools;

import prompt.input.IFlatFile;
import prompt.ui.DialogFactory;
import prompt.ui.FileFilter;

public class FlatFile extends SeqProperties implements IFlatFile {
	private final static long serialVersionUID = -6849794470754667700L;

	private String filename;

	private String format;

	private String alphabetName;

	protected final String formatDefault = "fasta";

	protected final String alphabetNameDefault = "protein";

	/**
	 * @param filename
	 *            full path and filename of the Protein-Fasta file to read
	 */
	public void init(String filename) throws Exception {
		Logger logger = Logger.getLogger(this.getClass());
		setFilename(filename);
		setFormat(formatDefault);
		setAlphabet(alphabetNameDefault);

		logger
				.debug("Just a filename supplied ==> Using now defaults: format='"
						+ format + "' and alphabet='" + alphabetName + "'");

		// logger.debug(
		// "Just a filename supplied ==> Using now defaults: format='"+format+"'
		// and alphabet='"+alphabetName+"'");

		if (!isFileOk()) {
			String errmsg = "File testing failed<br>Check if this is a <b>"
					+ format + "</b> file,<br>"
					+ "and contains only sequences of the type " + alphabetName;
			logger.warn(errmsg);
			throw new ClassCastException(errmsg);
		}
	}

	public void init(String filename, String format, String alphabetName)
			throws Exception {
		Logger logger = Logger.getLogger(this.getClass());
		setFilename(filename);
		setFormat(format);
		setAlphabet(alphabetName);
		logger.debug("Format='" + this.format + "' alphabetName='"
				+ this.alphabetName + "'");
		// logger.debug("Format='" + this.format + "' alphabetName='"
		// + this.alphabetName + "'");
		if (!isFileOk()) {
			String errmsg = "File testing failed<br>Check if this is a <b>"
					+ format + "</b> file,<br>"
					+ "and contains only sequences of the type " + alphabetName;
			logger.warn(errmsg);
			throw new ClassCastException(errmsg);
		}
	}

	private static String lastDirectory = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() {
		Logger logger = Logger.getLogger(this.getClass());
		boolean canceled = false;

		if (this.format == null) {
			Object[] format = { "fasta", "genbank", "embl", "swissprot" };
			String sf = (String) JOptionPane.showInputDialog(parentWin,
					"Type of file:", "Choose type", JOptionPane.PLAIN_MESSAGE,
					null, format, "FASTA");
			if (sf != null) {
				setFormat(sf);
			} else
				canceled = true;
		}

		if ((!canceled) && (this.alphabetName == null)) {
			Object[] alphabet = null;
			if (format.equals("fasta"))
				alphabet = new Object[] { "protein", "dna", "rna", "none" };
			else
				alphabet = new Object[] { "protein", "dna", "rna" };
			String sa = (String) JOptionPane.showInputDialog(parentWin,
					"Format of data in file:", "Choose format",
					JOptionPane.PLAIN_MESSAGE, null, alphabet, "Protein");
			if (sa != null) {
				setAlphabet(sa);
			} else
				canceled = true;
		}
		if ((!canceled) && (this.filename == null)) {
			JFileChooser fc = new JFileChooser(lastDirectory);
			fc.addChoosableFileFilter(new FileFilter(format));
			// System.getProperty("user.dir"));
			fc.setDialogTitle("Open " + format + " " + alphabetName
					+ " flatfile");
			int fileChooserMessage = fc.showOpenDialog(parentWin);
			switch (fileChooserMessage) {
			case JFileChooser.APPROVE_OPTION:
				if (fc.getSelectedFile() != null)
					setFilename(fc.getSelectedFile().getAbsolutePath());
				lastDirectory = fc.getSelectedFile().getAbsolutePath();
				break;
			case JFileChooser.CANCEL_OPTION: {
				canceled = true;
				break;
			}
			}
		}

		if (!canceled) {
			try {
				if (isFileOk()) {
					if (this.getDescription() == null)
						setDescription(DialogFactory
								.showInputDescriptionDialogFile(filename));

				} else {
					logger.debug("File testing failed");
					String errmsg = "File testing failed<br>Check if this is a <b>"
							+ format
							+ "</b> file,<br>"
							+ "and contains only sequences of the <b>type "
							+ alphabetName + "</b>";
					logger.error(errmsg);
					JMessage.showMessageDialog(null, errmsg,
							JOptionPane.INFORMATION_MESSAGE,
							JOptionPane.DEFAULT_OPTION);
					return;
				}
			} catch (Exception exc) {
				String errmsg = "Error processing file";
				logger.error(errmsg);
				JMessage.showMessageDialog(null, errmsg,
						JOptionPane.INFORMATION_MESSAGE,
						JOptionPane.DEFAULT_OPTION, exc);
				return;
			}
		} else {
			logger
					.debug("Not all necessary paramters for the FlatFile Interface could have been set");
			String errmsg = "Canceled by user";
			logger.info(errmsg);
			JMessage
					.showMessageDialog(null, errmsg,
							JOptionPane.INFORMATION_MESSAGE,
							JOptionPane.DEFAULT_OPTION);
			return;
		}

	}

	/*
	 * - Small datasets: here the actual data is gathered, e.g. read from
	 * pedant,xml or other file - Large datasets e.g. whole genome comparisons:
	 * create an interator which will be called by the Evaluation classes and
	 * reads the data on demand
	 */

	public boolean isFileOk() {
		Logger logger = Logger.getLogger(this.getClass());
		boolean ret = true;

		if (!this.alphabetName.equals("none")) { // only if we use a format
			SequenceIterator iter = null;
			try {

				BufferedReader br = new BufferedReader(new FileReader(filename));
				iter = (SequenceIterator) SeqIOTools.fileToBiojava(format,
						alphabetName, br);
				if (iter.hasNext()) {
					// SimpleSequence ssq = (SimpleSequence)
					// iter.nextSequence();
					iter.nextSequence();
				}
				br.close();
			} catch (Exception e) {
				logger.debug("File format '" + format + "' - '" + alphabetName
						+ "' does not fit to '" + filename + "' ");
				// logger.fatal("File format '" + format + "' - '" +
				// alphabetName
				// + "' does not fit to '" + filename + "' ");
				// logger.error("File format '"+format+"' - '"+alpabetName+"'
				// does
				// not fit to '"+filename+"' ");
				ret = false;

			}
		}

		return ret;
	}

	protected List cachedSequences = null;

	protected List cachedSequenceNames = null;

	/*
	 * hier koennten wir einen eigene Implementierung von List zureuck geben die
	 * die Daten LAZY list. Damit waeren keine Speicherprobleme mehr bei grossen
	 * Datenmengen. Da als Rueckgabe aber immer noch List gibt muss ansonsten
	 * kein weiterer Source code geandert werden.
	 */
	public List getSequences() {
		Logger logger = Logger.getLogger(this.getClass());
		if (cachedSequences != null)
			return cachedSequences;
		List seq = new LinkedList();

		if ((this.alphabetName.equals("none")) && (this.format.equals("fasta"))) {
			logger.debug("NEW File format '" + format + "' - '" + alphabetName
					+ "'");
			try {
				BufferedReader seqin = new BufferedReader(new FileReader(
						filename));
				String line = "";
				String block = "";

				while (seqin.ready()) {
					line = seqin.readLine();

					if (line.startsWith(">")) {
						if (!block.equals(""))
							seq.add(block);
						block = "";
					} else {
						line = line.replaceAll("\\W", "");
						block = block + line;
					}
					;
				}
				if (!block.equals("")) {
					seq.add(block);
				}
			} catch (Exception e) {
				logger.error(e);
			}
		} else {

			SequenceIterator iter = null;
			try {
				BufferedReader br = new BufferedReader(new FileReader(filename));
				iter = (SequenceIterator) SeqIOTools.fileToBiojava(format,
						alphabetName, br);
				while (iter.hasNext()) {
					SimpleSequence ssq = (SimpleSequence) iter.nextSequence();
					seq.add(ssq.seqString());
				}
				br.close();
			} catch (Exception e) {
				logger.error(e);
			}

		}
		cachedSequences = seq;
		return seq;
	}

	public List getSequencesNames() {
		if (cachedSequenceNames != null)
			return cachedSequenceNames;
		List seq = new LinkedList();
		Logger logger = Logger.getLogger(this.getClass());
		if ((this.alphabetName.equals("none")) && (this.format.equals("fasta"))) {
			logger.debug("NEW2 File format '" + format + "' - '" + alphabetName
					+ "'");
			try {
				BufferedReader seqin = new BufferedReader(new FileReader(
						filename));
				String line = "";

				while (seqin.ready()) {
					line = seqin.readLine();
					if (line.startsWith(">")) {
						try {
							String tmp = line.substring(1, line.length());
							int index1 = tmp.indexOf("|");
							if (index1 == -1) {
								index1 = tmp.indexOf(" ");
							}
							if (index1 == -1) {
								index1 = tmp.indexOf("\t");
							}
							if (index1 == -1) {
								seq.add(tmp);
							} else {
								String first = tmp.substring(0, index1);
								first = first.toUpperCase();
								seq.add(first);
								// logger.debug("'"+first+"'");
							}
						} catch (Exception e1) {
							logger.error(e1);
						}

					}
				}
			} catch (Exception e) {
				logger.error(e);
			}

		} else {
			SequenceIterator iter = null;
			try {
				BufferedReader br = new BufferedReader(new FileReader(filename));
				iter = (SequenceIterator) SeqIOTools.fileToBiojava(format,
						alphabetName, br);
				while (iter.hasNext()) {
					SimpleSequence ssq = (SimpleSequence) iter.nextSequence();
					seq.add(ssq.getName().trim().toUpperCase());
				}
				br.close();
			} catch (Exception e) {
				logger.error(e);
			}
		}
		cachedSequenceNames = seq;
		return seq;
	}

	public List getAnnotation() {
		// public List getAnnotation(String format) {
		List seq = new LinkedList();
		SequenceIterator iter = null;
		try {
			BufferedReader br = new BufferedReader(new FileReader(filename));
			iter = (SequenceIterator) SeqIOTools.fileToBiojava(format,
					alphabetName, br);
			while (iter.hasNext()) {
				SimpleSequence ssq = (SimpleSequence) iter.nextSequence();
				seq.add(ssq.getAnnotation());
			}
			br.close();
		} catch (Exception e) {
			Logger logger = Logger.getLogger(this.getClass());
			logger.error(e);
		}
		return seq;
	}

	public String getMetaData() {
		StringBuffer sb = new StringBuffer();
		sb.append("<html><table>");

		sb.append("<tr><td> load from file </td>");
		sb.append("<td>" + filename + "</td></tr>");

		sb.append("<tr><td> format </td>");
		sb.append("<td>" + format + "</td></tr>");

		sb.append("<tr><td> symbol alphabet name </td>");
		sb.append("<td>" + alphabetName + "</td></tr>");

		sb.append("</html></table>");
		JMessage.showMessageDialog(null, sb.toString(),
				JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

		return sb.toString();
	}

	/**
	 * 
	 * @uml.property name="format"
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.IFlatFile#setFormat(java.lang.String)
	 */
	public boolean setFormat(String pFormat) {
		this.format = pFormat;
		// TODO check if format-parameter is supported
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.IFlatFile#setAlphabet(java.lang.String)
	 */
	public boolean setAlphabet(String pAlphabet) {
		this.alphabetName = pAlphabet;
		return true;
	}

	/**
	 * 
	 * @uml.property name="filename"
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.IFlatFile#setFilename(java.lang.String)
	 */
	public void setFilename(String pFilename) {
		this.filename = pFilename;

	}

	/**
	 * 
	 * @uml.property name="filename"
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.IDataFile#getFilename()
	 */
	public String getFilename() {
		return filename;
	}

	public Component parentWin = null;

	public void setParentWindow(Component win) {
		this.parentWin = win;
	}

	public Iterator iterator() {
		return new SeqIterator(alphabetName, format);
	}

	public class SeqIterator implements Iterator {
		Logger logger = Logger.getLogger(this.getClass());

		String alphabetName, format;

		BufferedReader seqin;

		SequenceIterator bioJIterator;

		// indicates if the next element has already been used by the hasNext()
		// method
		boolean cached = false;

		// contain the name and sequence if hasNext() has been called
		String cachedName, cachedSequence;

		public SeqIterator(String alphabetName, String format) {
			this.alphabetName = alphabetName;
			this.format = format;

			try {
				seqin = new BufferedReader(new FileReader(filename));
			} catch (FileNotFoundException e) {
				logger.error("File not found (" + filename + ")", e);
			}
			if (!(this.alphabetName.equals("none"))
					&& (this.format.equals("fasta"))) {
				bioJIterator = null;

				BufferedReader br;
				try {
					br = new BufferedReader(new FileReader(filename));
					bioJIterator = (SequenceIterator) SeqIOTools.fileToBiojava(
							format, alphabetName, br);
				} catch (FileNotFoundException e) {
					logger.error("File not found (" + filename + ")", e);
				} catch (BioException e) {
					logger.error("Wrong format/alphabet combination", e);
				}

			}
		}

		public boolean hasNext() {
			String name = null, sequence = null;
			if ((this.alphabetName.equals("none"))
					&& (this.format.equals("fasta"))) {
				try {
					if (!seqin.ready()) {
						return false;
					}
				} catch (IOException e) {
					logger.error("IO error", e);
				}
				String line = "";
				try {
					line = seqin.readLine();
				} catch (IOException e) {
					logger.error("IO error", e);
				}
				if (line.startsWith(">")) {

					String tmp = line.substring(1, line.length());
					int index1 = tmp.indexOf("|");
					if (index1 == -1) {
						index1 = tmp.indexOf(" ");
					}
					if (index1 == -1) {
						index1 = tmp.indexOf("\t");
					}
					if (index1 == -1) {
						name = tmp;
					} else {
						String first = tmp.substring(0, index1);
						first = first.toUpperCase();
						name = first;
					}

				}
				String block = "";

				if (line.startsWith(">")) {
					if (!block.equals(""))
						sequence = block;
					block = "";
				} else {
					line = line.replaceAll("\\W", "");
					block = block + line;
				}

				if (!block.equals("")) {
					sequence = block;
				}

			} else {
				SimpleSequence bioJSeq = null;
				try {
					if (bioJIterator.hasNext()) {
						bioJSeq = (SimpleSequence) bioJIterator.nextSequence();
					} else {
						return false;
					}
				} catch (NoSuchElementException e) {
					System.out.println("NO SUCHELEMEBNT");
					e.printStackTrace();
				} catch (BioException e) {
					e.printStackTrace();
				}
				name = bioJSeq.getName();
				sequence = bioJSeq.seqString();
			}

			if (name != null && sequence != null) {
				cachedName = name;
				cachedSequence = sequence;
				cached = true;
				return true;
			}
			return false;

		}

		public Object next() {
			String name = null, sequence = null;
			if (cached) {
				cached = false;
				return new Object[] { cachedName, cachedSequence };
			}
			if ((this.alphabetName.equals("none"))
					&& (this.format.equals("fasta"))) {
				try {
					String line = "";
					line = seqin.readLine();
					if (line.startsWith(">")) {
						try {
							String tmp = line.substring(1, line.length());
							int index1 = tmp.indexOf("|");
							if (index1 == -1) {
								index1 = tmp.indexOf(" ");
							}
							if (index1 == -1) {
								index1 = tmp.indexOf("\t");
							}
							if (index1 == -1) {
								name = tmp;
							} else {
								String first = tmp.substring(0, index1);
								first = first.toUpperCase();
								name = first;
							}
						} catch (Exception e1) {
							e1.printStackTrace();
						}
					}
					String block = "";

					if (line.startsWith(">")) {
						if (!block.equals(""))
							sequence = block;
						block = "";
					} else {
						line = line.replaceAll("\\W", "");
						block = block + line;
					}

					if (!block.equals("")) {
						sequence = block;
					}
				} catch (Exception e) {

				}
			} else {
				SimpleSequence bioJSeq = null;
				try {
					bioJSeq = (SimpleSequence) bioJIterator.nextSequence();
				} catch (NoSuchElementException e) {
					e.printStackTrace();
				} catch (BioException e) {
					e.printStackTrace();
				}
				name = bioJSeq.getName();
				sequence = bioJSeq.seqString();
			}
			return new Object[] { name, sequence };

		}

		public void remove() {
			throw new UnsupportedOperationException();
		}

	}

}
