/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SeqProperties.java,v 1.12 2006/04/12 12:33:16 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;



import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.symbol.IllegalAlphabetException;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.bio.symbol.SymbolPropertyTable;
import org.biojava.bugfixes.IsoelectricPointCalc;
import org.biojava.bugfixes.MassCalc;

import prompt.engine.genericxml.GenericHelper;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.ISeqProperties;
import tschmidt.scales.HphobKyteDoolite;
import tschmidt.scales.Scale;
import tschmidt.utils.Db;


/**
 * @author schmidt
 *  
 */
public abstract class SeqProperties extends Input implements IGenericXML,ISeqProperties {
	private final static long serialVersionUID = -6849794470754667700L;

	/* (non-Javadoc)
     * @see prompt.input.IGenericXML#getLabel()
     */
    public String getLabel() {
        return this.getDescription();
    }

   
    /* (non-Javadoc)
     * @see prompt.input.IGenericXML#getProperties()
     */
    public IGenericProperty[] getProperties() {
        /*for all seqproperties (of the ISequence Interface)
          generate a proxy class that will call this stuff
        */
         
        IGenericProperty[] ret = new IGenericProperty[8];
        ret[0] = new SeqPropertiesProxy(this,"setdef",IGenericProperty.SETDEF);
        ret[1] = new SeqPropertiesProxy(this,"mass",IGenericProperty.NUMERIC);
        ret[2] = new SeqPropertiesProxy(this,"length",IGenericProperty.NUMERIC);
        ret[3] = new SeqPropertiesProxy(this,"pI",IGenericProperty.NUMERIC);
        ret[4] = new SeqPropertiesProxy(this,"pIawayFromNeutral",IGenericProperty.NUMERIC);
        ret[5] = new SeqPropertiesProxy(this,"HydrophobicityAvg",IGenericProperty.NUMERIC);
        ret[6] = new SeqPropertiesProxy(this,"HydrophobicitySum",IGenericProperty.NUMERIC);
        ret[7] = new SeqPropertiesProxy(this,"AlternatingHPhobHPhil",IGenericProperty.NUMERIC);
        
        /*
        Class superClazz = super.getClass();
        if (IGenericXML.class.isAssignableFrom(superClazz)) {
            ret = mergePropertyArrays(super.getProperties(),ret);
        }*/
        return ret;
      
    }
    
    public  IGenericProperty getProperty(String propertyId) {
    	IGenericProperty[] list = getProperties();
    	if (list==null) return null;
    	if (list.length==0) return null;
    	for (int i = 0; i < list.length; i++) {
			if (list[i]!=null) {
				if (list[i].getId()!=null) {
					if (list[i].getId().equalsIgnoreCase(propertyId)) return list[i];
				}
			}
		}
    	return null;
    }

    class SeqPropertiesProxy implements IGenericProperty{
        ISeqProperties backref;
        int type = IGenericProperty.YETUNKOWN;
        String method = "unknown";
        
        public SeqPropertiesProxy(ISeqProperties backref,String method,int type) {
            this.backref=backref;
            this.method=method;
            this.type=type;      
        }

        public int getType() {
            return type;
        }

        public String getId() {
            return method;
        }

        

        
        public HashMap getValues() {
            /*je nachdem was fuer eine methode rufe eine andere
              Methode des SeqProperties Objeckts auf das als backref
              bekannt ist.
            */
            try {
            if (method.equals("setdef")) {
                return GenericHelper.makeHash(backref,backref.getSequences());
            }
            if (method.equals("length")) {
                return GenericHelper.makeHash(backref,backref.getLength());
            }
            if (method.equals("mass")) {
                return GenericHelper.makeHash(backref,backref.getMass());
            }
            if (method.equals("pI")) {
                return GenericHelper.makeHash(backref,backref.getPI());
            }
            if (method.equals("pIawayFromNeutral")) {
                return GenericHelper.makeHash(backref,backref.getPIawayFromNeutral());
            }
            if (method.equals("HydrophobicityAvg")) {
                Scale scale = new HphobKyteDoolite();
                return GenericHelper.makeHash(backref,backref.getHydrophobicityAvg(scale));
            }           
            if (method.equals("HydrophobicitySum")) {
                return GenericHelper.makeHash(backref,backref.getHydrophobicitySum(new HphobKyteDoolite()));
            }   
            if (method.equals("AlternatingHPhobHPhil")) {
                return GenericHelper.makeHash(backref,backref.getAlternatingHPhobHPhil());
            }

            } catch (Exception e) {
                Logger logger = Logger.getLogger(SeqPropertiesProxy.class);
                logger.fatal("error reading property: ",e);
            }
           
            return null;
        }
        
    }
    
    
    /*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.ISeqProperties#getLength()
	 */
	public List getLength() {
		List ret = new LinkedList();
		List list = getSequences();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null) {
				String seqTmp = seq.replaceAll("\\W", "");
				ret.add(new Integer(seqTmp.length()));
			}else
				ret.add(Double.NaN); 
		}
		return ret;
	}

	/*
	 * protected static MassCalc mc = new MassCalc(SymbolPropertyTable.AVG_MASS,
	 * true);
	 */
	protected static IsoelectricPointCalc ic = new IsoelectricPointCalc();

	/**
	 * Calculates the Mass of the peptide in Daltons. Using the average Isotope
	 * Mass
	 * 
	 * @param protein
	 *            the peptide
	 * @throws IllegalSymbolException
	 *             if <code>protein</code> is not a protein
	 * @return the mass
	 */
	public static double mass(String protein) throws IllegalSymbolException {
		double mass = 0.0;
		if ((protein==null) || (protein.length()==0)) return Double.NaN;
		protein = protein.replaceAll("\\W", "");
		SymbolList proteinSym = (SymbolList) ProteinTools
				.createProtein(protein);
		if ((proteinSym.getAlphabet() == ProteinTools.getTAlphabet())
				|| (proteinSym.getAlphabet() == ProteinTools.getAlphabet())) {

			mass = MassCalc.getMass(proteinSym, SymbolPropertyTable.AVG_MASS,
					true);

		} else {
			// not a peptide
			throw new IllegalSymbolException("in seq '" + protein + "'");
		}
		return mass;
	}

	/**
	 * Calculates the isoelectric point assuming a free NH and COOH
	 * 
	 * @param protein
	 *            the peptide
	 * @throws IllegalAlphabetException
	 *             if <code>protein</code> is not a peptide
	 * @throws BioException
	 * @return double the PI <br>
	 * 
	 * The isoelectric point (or pI) of a protein is the pH at which the protein
	 * has an equal number of positive and negative charges. This attribute may
	 * be determined by isoelectric focusing (IEF), which is also the first
	 * dimension of 2D-PAGE (two-dimensional polyacrylamide gel
	 * electrophoresis). <br>
	 * <br>
	 * As a rule, isoelectric point is not a very good tool for identifying a
	 * protein. The experimental values determined by electrophoresis is not
	 * very accurate and all sorts of posttranslational modifications can
	 * produce a difference between experimental pI and theoretical pI (which
	 * the tools use) of as much as 2 whole numbers! Allowing that much error
	 * into the match makes it completely useless. <br>
	 * <br>
	 * What might work is predicting isoelectric point values. There will
	 * certainly still be large differences but if you have an idea what your
	 * proteins are and are using a larger pH gradient for electrophoresis, it
	 * might help narrow the identity of spots and bands. <br>
	 */

	public static double pI(String protein) throws IllegalAlphabetException,
			BioException {
		if (protein==null) return Double.NaN;
		double pI = 0.0;
		protein = protein.replaceAll("\\W", "");

		SymbolList proteinSym = (SymbolList) ProteinTools
				.createProtein(protein);

		pI = ic.getPI(proteinSym, true, true);
		return pI;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.ISeqProperties#getMass() double mass =
	 *      calcMass.mass(syms); double pI = calcMass.pI(syms);
	 */
	public List getMass() throws IllegalSymbolException {
		List ret = new LinkedList();
		List list = getSequences();

		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null)
			ret.add(new Double(mass(seq)));else
				ret.add(Double.NaN);
		}
		return ret;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.ISeqProperties#getPI()
	 */
	public List getPI() throws IllegalAlphabetException, BioException {
		List ret = new LinkedList();
		List list = getSequences();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null) ret.add(new Double(pI(seq))); else
				ret.add(Double.NaN);
		}
		return ret;
	}

	public List getPIawayFromNeutral() throws IllegalAlphabetException, BioException {
		List ret = new LinkedList();
		List list = getSequences();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null) {
				
				ret.add(new Double(Math.abs(7d-pI(seq)))); }
			else
				ret.add(Double.NaN);
		}
		return ret;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.ISeqProperties#getHydrophobicityAvg(tschmidt.scales.Scale)
	 */
	public List getHydrophobicityAvg(Scale scale) {
		List ret = new LinkedList();
		List list = getSequences();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null) {
				String seqTmp = seq.replaceAll("\\W", "");
				ret.add(new Float(scale.avgValue(seqTmp)));
			}else
				ret.add(Double.NaN);
		}
		return ret;
	}

	/**
	 * Tests whether a AA at a given SeqPosition is hydrophobic
	 * 
	 * @param seq
	 * @param pos
	 * @return
	 */
	 boolean isHydrophobic(String seq, int pos) {
		boolean ret = false;
		
		seq = seq.toUpperCase();

		char[] hphobic = { 'A', 'C', 'F', 'G', 'I', 'L', 'M', 'P', 'V', 'W','Y' };
		for (int i = 0; i < hphobic.length; i++) {
			if (seq.charAt(pos) == hphobic[i]) {
				ret = true;
			}
		}
		return ret;
	}

	//
	/**
	 * Gives a list with the number of stretches of alternating
	 * hydrophopic/hydrophilic residues of a sequence list
	 * 
	 * @return number of stretches
	 */
	public List getAlternatingHPhobHPhil() {
		List ret = new LinkedList();
		List list = getSequences();
		int amount_consec = 0;
		int counter_consec = 1;

		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null) {
				seq = seq.replaceAll("\\W", "");	
			seq = seq.toUpperCase();
			for (int i = 1; i < seq.length(); i++) {
				if ((isHydrophobic(seq, i) && isHydrophobic(seq, i - 1))
						|| (!isHydrophobic(seq, i) && !isHydrophobic(seq, i - 1))) {
					if (counter_consec >= 5) {
						amount_consec++;
						//logger.debug("Check"+(i-counter_consec)+":"+seq.substring(i-counter_consec,i));
					}
					counter_consec = 1;
				} else {
					counter_consec++;
				}
			}
			ret.add(new Integer(amount_consec));
			amount_consec = 0;
			}
		}
		return ret;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.ISeqProperties#getHydrophobicitySum(tschmidt.scales.Scale)
	 */
	public List getHydrophobicitySum(Scale scale) {
		List ret = new LinkedList();
		List list = getSequences();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			if (seq!=null) {
				String seqTmp = seq.replaceAll("\\W", "");
				 ret.add(new Float(scale.sumValue(seqTmp)));
			}
			else
				ret.add(Double.NaN);
		}
		return ret;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.proj.input.IInput#getSize()
	 */
	public int getSize() {
		List list = getSequencesNames();
		if (list == null)
			return 0;

		int i = 0;
		for (Iterator iter = list.iterator(); iter.hasNext();) {
		//	String element = (String) iter.next();
			iter.next();
			i++;
		}
		return i;
	}

	/**
	 * Returns the number of Beta-aggregation Stretches of a a single sequence
	 * out of a sequence List. For this TANGO is used http://tango.embl.de/
	 * 
	 * @return
	 * @throws IOException
	 * @throws NumberFormatException
	 */
	public List getBetaAggregationStretches() throws NumberFormatException,
			IOException {
		checkTango();
        Logger logger = Logger.getLogger(this.getClass());
		List ret = new LinkedList();
		List tmp = new LinkedList();
		List list = getSequences();
		
		int index = 1;
		Connection conn=null;
		Statement stmt = null;
		try {
			conn = Db.connectDb("abundance_final");
			stmt = conn.createStatement();
		} catch (SQLException e3) {
            logger.error(e3);
		}
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();
			String[] cut = seq.split(";");
			logger.debug(cut[1]);
			try {
				/*logger.debug("TangoResult"
						+ cut[1]
								+ " nt=N ct=N ph=7.4 te=310 io=0.05 tf=0 stab=-10 seq="
								+ cut[0]);
			*/

					tmp = callTango("TangoResult"
							+ cut[1]
							+ " nt=N ct=N ph=7 te=298.15 io=0.02 tf=0 stab=-10 seq="
							+ cut[0]);
				try{
					File tango_output = new File((String) tmp.get(0));
					BufferedReader out = null;
					try {
						out = new BufferedReader(new FileReader(tango_output));
					} catch (FileNotFoundException e1) {
                        logger.error(e1);
					}
					//Fetch just the beta-score for a given residue and count
					// the consecutive stretches where
					//beta score >= 5
					String thisLine = "";
					String[] columns;

					int counter_consec = 0;
					int amount_consec = 0;

					while ((thisLine = out.readLine()) != null) {
						//logger.debug(thisLine);
						columns = thisLine.split(",");
						//logger.debug("beta columns:"+ columns[5]);

						if (Double.parseDouble(columns[5]) > 5) {
							counter_consec++;
						} else {
							if (counter_consec >= 5) {
								amount_consec++;
							}
							counter_consec = 0;
						}
					}
					//logger.debug("beta-aggregating stretches:
					// "+amount_consec);
					try{
						String qry = "UPDATE rep SET tango_beta_stretches = "+amount_consec+" where rep.id ="+ cut[1]+";";
						stmt.executeUpdate(qry);
					}
					catch(Exception e){
                        logger.error(e);
					}
					ret.add(new Integer(amount_consec));
					amount_consec = 0;
					tango_output.delete();
					index++;
				}catch(Exception e){
					//logger.warn(e);
					logger.warn("TANGO wasn't able to handle this sequence.");
					logger.warn("tango_beta_stretches set to -2");
					String qry = "UPDATE rep SET tango_beta_stretches = -2 where rep.id ="+ cut[1]+";";
					try {
						stmt.executeUpdate(qry);
					} catch (SQLException e1) {
                        logger.error(e1);
					}
				}
					
			} catch (IOException e) {
                logger.error(e);
			}
		}
		return ret;
	}

	
	/**
	 * Calls TANGO with the parameters given by the input String
	 * 
	 * @param input
	 * @return list: first element is the TANGO Output File,Second element in
	 *         the List is the TANGO console output
	 * @throws IOException
	 */
	public static List callTango(String input) throws IOException {
		List ret = new LinkedList();
		String result = "";

		String[] cmds;
		String[] cmd_Array = input.split("\\s");

		cmds = new String[(cmd_Array.length + 1)];
		cmds[0] = System.getProperty("tango2_1.gexe");

		for (int i = 1; i <= cmd_Array.length; i++) {
			cmds[i] = cmd_Array[i - 1];
		}
		//logger.debug(input);
		try {
			Process p = Runtime.getRuntime().exec(cmds, null,
					new File(System.getProperty("java.io.tmpdir")));
			BufferedReader r = new BufferedReader(new InputStreamReader(p
					.getInputStream()));
			BufferedWriter w = new BufferedWriter(new OutputStreamWriter(p
					.getOutputStream()));
			w.close();
			String thisLine;

			while ((thisLine = r.readLine()) != null) {
				//logger.debug("TANGO>: '" + thisLine + "'");
				result += thisLine;
			}
		} catch (IOException e) {
            logger.error(e);
		}

		ret.add(System.getProperty("java.io.tmpdir") + File.separator + cmds[1]
				+ ".OUT");
		ret.add(result);
		logger.debug(".");
		return ret;
	}

	/**
	 * Checks the TANGO SystemProperties
	 */
	private static Logger logger = Logger.getLogger(SeqProperties.class);

	public static void checkTango() {

		logger.debug("CHECK TANGO");
		//logger.debug(showSystemProperties());

		//Check if all properties are set up
		if (System.getProperty("tango2_1.gexe") == null) {
			logger
					.warn("System-property 'tango2_1.gexe' is NOT SET to an executable binary of TANGO");
			String os = System.getProperty("os.name");
			if (os.matches(".*?Windows.*")) {
				//FIXME: hier noch ausfallsicherer coden!!!
				System.setProperty("tango2_1.gexe", "..\\tango\\tango2_0.exe");

			} else {
				System.setProperty("tango2_1.gexe",
						"/home/proj/thorsten/necessary/Tango/tango2_1");
			}
			logger.debug("'tango2_1.gexe' has been set to '"
					+ System.getProperty("tango2_1.gexe") + "'");
		}

	}


  /*  public int getType() {
        // TODO Auto-generated method stub
        return 0;
    }


    public String getId() {
        // TODO Auto-generated method stub
        return null;
    }


    public HashMap getValues() {
        // TODO Auto-generated method stub
        return null;
    }*/

}
