/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SwissProtFlatFileReader.java,v 1.9 2006/04/06 11:56:09 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes.swissProt.internals;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.input.classes.FlatFile;
import prompt.input.swissProt.ISwissProtFileReader;

public class SwissProtFlatFileReader implements ISwissProtFileReader,
		Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	Logger logger = Logger.getLogger(this.getClass().getName());

	// true, if calculations shall be printed to console
	private boolean print = false;

	private Description de;

	private OrganismSpecies os;

	private Organelle og;

	private OrganismClassification oc;

	private Reference rx;

	private Keywords kw;

	private FeatureTable ft;

	private Domain dom;

	private FlatFile flat;

	private List<String> seqIds;

	private List<String> seqs;

	private List<String> seqDes;

	public SwissProtFlatFileReader() {
		de = new Description();
		os = new OrganismSpecies();
		og = new Organelle();
		oc = new OrganismClassification();
		rx = new Reference();
		kw = new Keywords();
		ft = new FeatureTable();
		dom = new Domain();
	}

	public int askUser(String msg, String title) {
		Logger logger = Logger.getLogger(this.getClass().getName());
		/*
		 * if (parentWin != null) logger.debug("Parent win not null"); else
		 * logger.debug("Parent win IS NULL");
		 */
		return JOptionPane.showConfirmDialog(null, msg, title,
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);

	}

	public void parseSwissProtFileArray(ArrayList<File> proteinGroupArray)
			throws Exception {

		seqIds = new LinkedList();
		seqs = new LinkedList();
		seqDes = new LinkedList();

		// regular expression pattern
		Pattern regexpID = Pattern.compile("^ID {3}([A-Z|[0-9]|_]+).*$");
		Pattern regexpDE = Pattern.compile("^DE.*?\\(EC(.*?)\\)(.*)$");
		Pattern regexpOS = Pattern
				.compile("^OS +([^\\(].+?[^\\)])[\\.|\\(|\\,].*$");
		Pattern regexpOG = Pattern.compile("^OG(.+?[^ ])[\\.|\\,| ].*$");
		Pattern regexpOC = Pattern.compile("^OC(.*?)\\.?$");
		Pattern regexpMEDLINE = Pattern.compile("^RX.*MEDLINE=(.*?);.*$");
		Pattern regexpPubMed = Pattern.compile("^RX.*PubMed=(.*?);.*$");
		Pattern regexpKW = Pattern.compile("^KW(.*?)\\.?$");
		Pattern regexpFT = Pattern.compile("^FT {3}([A-Z|_]+).*$");
		Pattern regexpDomains = Pattern.compile("^DR {3}(.*)$");

		// parse every swissprot file contained in proteinGroupArray
		for (int i = 0; i < proteinGroupArray.size(); i++) {
			// use FlatFile to read basic stuff like sequences, ids und
			// descriptions
			flat = new FlatFile();

			String fname = proteinGroupArray.get(i).getAbsolutePath();
			flat.setDescription(fname);
			logger.debug("parse now '" + fname + "'");
			try {
				flat.init(fname, "swissprot", "protein");
				flat.run();
				List tmp = flat.getSequencesNames();
				logger.debug("found " + tmp.size() + " ids");
				for (Object object : tmp) {
					seqIds.add((String) object);
				}

				tmp = flat.getSequences();
				logger.debug("found " + tmp.size() + " sequences");
				for (Object object : tmp) {
					seqs.add((String) object);
				}

				/*
				 * tmp = flat.getAnnotation(); logger.debug("found
				 * "+tmp.size()+" sequence descriptions"); for (Object object :
				 * tmp) { seqDes.add((String) object); }
				 */

			} catch (Exception e1) {
				String errmsg = "<html>could not parse file '" + fname
						+ "'<br>" + e1.getLocalizedMessage() + "<br>"
						+ "Do you really want to continue?</html>";
				logger.error(errmsg);
				if (askUser(errmsg, "PARSING ERROR") == JOptionPane.NO_OPTION) {
					throw new Exception(e1);
				}
			}

			// other variables
			boolean matchOG = true;
			String mergeOC = "";
			Hashtable featureTableKeyNameHashtable = new Hashtable();

			// parse swissprot file
			// Scanner scan = new
			// Scanner(proteinGroupArray[i]).useDelimiter(System.getProperty("line.separator"));
			try {

				BufferedReader in = new BufferedReader(new FileReader(
						proteinGroupArray.get(i)));
				// print file identier to console
				Matcher id = regexpID.matcher(in.readLine());
				if (id.matches()) {
					if (print) {
						logger.debug("ID\t" + id.group(1));
					}
				}

				while (in.ready()) {
					String line = in.readLine();

					// DE LINE
					Matcher description = regexpDE.matcher(line);
					if (description.matches()) {
						// possibly more than one EC number in one line
						while (description.matches()) {
							// add EC number to Description object
							de.newDEEntry("EC " + description.group(1).trim());
							// print EC number to console
							if (print)
								logger.debug("DE\t"
										+ description.group(1).trim());
							// try to match pattern in remaining line that must
							// start with 'DE'
							description = regexpDE.matcher("DE"
									+ description.group(2));
						}
						continue;
					}

					// OS LINE
					Matcher organismSpecies = regexpOS.matcher(line);
					if (organismSpecies.matches()) {
						// add organism species to OrganismSpecies object
						os.newOSEntry(organismSpecies.group(1).trim());
						// print organism species to console
						if (print)
							logger.debug("OS\t"
									+ organismSpecies.group(1).trim());
						continue;
					}

					// OG LINE
					if (matchOG) {
						Matcher organelle = regexpOG.matcher(line);
						if (organelle.matches()) {
							// add organelle to Organelle object
							og.newOGEntry(organelle.group(1).trim());
							// print organell to console
							if (print)
								logger
										.debug("OG\t"
												+ organelle.group(1).trim());
							// file possibly contains more than one Plasmid, not
							// important for us
							matchOG = false;
							continue;
						}
					}

					// OC LINE
					Matcher organismClassification = regexpOC.matcher(line);
					if (organismClassification.matches()) {
						mergeOC = mergeOC
								+ organismClassification.group(1).trim()
										.replaceAll(" ", "").replaceAll("\\;",
												".");
						continue;
					}

					// RX LINE
					Matcher medline = regexpMEDLINE.matcher(line);
					if (medline.matches()) {
						// add MEDLINE identifier to Reference object
						rx.newMedlineEntry(medline.group(1).trim());
						// print MEDLINE identifier to console
						if (print)
							logger.debug("MEDLINE\t" + medline.group(1).trim());
					}
					Matcher pubMed = regexpPubMed.matcher(line);
					if (pubMed.matches()) {
						// add PubMed identifier to Reference object
						rx.newPubMedEntry(pubMed.group(1).trim());
						// print PubMed identifier to console
						if (print)
							logger.debug("PubMed\t" + pubMed.group(1).trim());
					}

					// KW LINE
					Matcher keywords = regexpKW.matcher(line);
					if (keywords.matches()) {
						String[] keywordArray = keywords.group(1).trim()
								.replaceAll(" ", "").split("\\;");
						// add keywords to Keyword object
						kw.newKWEntry(keywordArray);
						// print keywords to console
						for (int j = 0; j < keywordArray.length; j++) {
							if (print)
								logger.debug("KW\t" + keywordArray[j]);
						}
						continue;
					}

					// FT Line
					Matcher featureTable = regexpFT.matcher(line);
					if (featureTable.matches()) {
						// maintain a unique key name hashtable
						featureTableKeyNameHashtable.put(featureTable.group(1)
								.trim(), new Integer(0));// zero cause we've
						// to put an object
						// in
					}
					// Domains
					Matcher domain = regexpDomains.matcher(line);
					if (domain.matches()) {
						String[] DrLine = ((String) domain.group(1)).split(";");
						if (DrLine[0].equals("Pfam")
								|| DrLine[0].equals("InterPro")
								|| DrLine[0].equals("PROSITE")
								|| DrLine[0].equals("SMART")
								|| DrLine[0].equals("PRINTS")) {
							dom.newDomainEntry(DrLine[1]);
						}
					}

				}
				// swissprot file completely read in
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			// process further objects

			// OC Line
			// add organism classification to OrganismClassification object
			oc.newOCEntry(mergeOC);
			// print organism classification to console
			if (print)
				logger.debug("OC\t" + mergeOC);

			// FT LINE
			String[] featureTableKeyNameArray = new String[featureTableKeyNameHashtable
					.keySet().size()];
			featureTableKeyNameHashtable.keySet().toArray(
					featureTableKeyNameArray);
			// add key names to FeatureTable object
			ft.newFTEntry(featureTableKeyNameArray);
			// print unique feature table key names to console
			for (int j = 0; j < featureTableKeyNameArray.length; j++) {
				if (print)
					logger.debug("FT\t" + featureTableKeyNameArray[j]);
			}

		}
		Logger logger = Logger.getLogger(this.getClass().getName());

		logger.info("Reading input data done!!!");

	}

	public Description getDE() {
		return de;
	}

	public FeatureTable getFT() {
		return ft;
	}

	public Keywords getKW() {
		return kw;
	}

	public OrganismClassification getOC() {
		return oc;
	}

	public Organelle getOG() {
		return og;
	}

	public OrganismSpecies getOS() {
		return os;
	}

	public Reference getRX() {
		return rx;
	}

	public Domain getDOM() {
		return dom;
	}

	public List getSequences() {
		return seqs;
	}

	public List getSequencesNames() {
		return seqIds;
	}

	public List getAnnotation() {
		return seqDes;
	}

	public Iterator getIterator() {
		return flat.iterator();
	}

	/*
	 * public static void main(String[] args) throws FileNotFoundException{
	 * 
	 * File testfile1 = new
	 * File("/home/users/siebert/SwissProt/MMP3_HUMAN.txt"); File testfile2 =
	 * new File("/home/users/siebert/SwissProt/CO2A1_HUMAN.txt"); File[]
	 * testfileArray = new File[]{testfile1, testfile2}; SwissProtFlatFileReader
	 * readin = new SwissProtFlatFileReader();
	 * readin.parseSwissProtFileArray(testfileArray); // Organelle og =
	 * readin.getOG(); // Hashtable tableOG = og.getOGSummary(); }
	 */

	public ArrayList getAC() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setAC(ArrayList acNr) {
		// TODO Auto-generated method stub

	}

	
	
}
