/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: DynamicLoading.java,v 1.5 2006/02/09 17:16:02 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.util;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Logger;

import prompt.engine.IEngine;
import prompt.engine.IEngine1;
import prompt.engine.IEngine2;
import prompt.engine.IEngineN;
import prompt.engine.IEngineResults;
import prompt.input.IInput;
import prompt.statistic.ITest;
import prompt.vis.IVisualisation;


/**
 * @author schmidt 1.6 changed to interface design, please test it
 */

public class DynamicLoading {
    public static Logger logger = Logger.getLogger(DynamicLoading.class);

    public static final int CLASSNOTFOUND = 100;
    
    public static final int UNKNOWNTYPE = 0;

    public static final int INPUT = 10;

    public static final int ENGINE1 = 20;

    public static final int ENGINE2 = 21;

    public static final int ENGINEN = 22;
    
    public static final int ENGINE0 = 23;

    public static final int TEST = 30;

    public static final int VIS = 40;

    public static Class getClass(String className) {
        Class classToRun = null;
        try {
            classToRun = Class.forName(className);
        } catch (ExceptionInInitializerError e) {
            logger.debug("the initialization failed loading class '" + className + "'.", e);
        } catch (LinkageError e) {
            logger.debug("the linkage failed loading class '" + className + "'.", e);
        } catch (ClassNotFoundException e) {
            logger.debug("the class '" + className + "' could not be found in the classpath. ");
        }
        if (classToRun == null) {
            logger.debug("class '" + className + "' loading not successful  :-(");
        }
        return classToRun;
    }

    
    public static boolean implementsInterface(Class myClass, String interfaceName) {
        // test if 'myClass' implments the interface named 'interfaceName'
        Class[] interfaces = myClass.getInterfaces();
        for (int i=0; i<interfaces.length; i++) {
            if (interfaces[i].getName().equals(interfaceName)) {
                return true;
            }
        }
        
        // since 'myClass' doesn't implment 'interfaceName', lets recursively look
        // for it's superclasses
        Class superClass = myClass.getSuperclass();
        if (superClass!=null) {
            return implementsInterface(superClass, interfaceName);
        } else {
            return false;
        }
    }
    
    public static int getType(Class clazz) {
        if (clazz == null)
            return UNKNOWNTYPE;
        if (IEngineN.class.isAssignableFrom(clazz))
            return ENGINEN;
        if (IEngine2.class.isAssignableFrom(clazz))
            return ENGINE2;
        if (IEngine1.class.isAssignableFrom(clazz))
            return ENGINE1;

        if (ITest.class.isAssignableFrom(clazz))
            return TEST;
        if (IVisualisation.class.isAssignableFrom(clazz))
            return VIS;
        if (IInput.class.isAssignableFrom(clazz))
            return INPUT;
        if (IEngine.class.isAssignableFrom(clazz))
            return ENGINE0;
        
        return UNKNOWNTYPE;
    }

    public static int getType(String className) {
        Class classToRun = null;
        try {
            classToRun = Class.forName(className);
        } catch (ExceptionInInitializerError e) {
            logger.debug("error in the static method of the class '" + className + "'.", e);
        } catch (LinkageError e) {
            logger.debug("the linkage failed loading the class '" + className + "'.", e);
        } catch (ClassNotFoundException e) {
            logger.debug("the class '" + className + "' could not be found in the classpath. ");
        }
        if (classToRun == null) {
            logger.debug("class '" + className + "' loading not successful  :-(");
            return CLASSNOTFOUND;
        }
        if (classToRun == null)
            return UNKNOWNTYPE;
        return getType(classToRun);
    }

    public static void main(String[] args) {
        logger.debug(getType("prompt.engine.seqproperties.AaDistribution"));

    }

    public static Method findMethod(Class clazz, String methodName) {
        Method[] methods = clazz.getMethods();
        for (int i = 0; i < methods.length; i++) {
            if (methods[i].getName().equals(methodName))
                return methods[i];
        }
        return null;
    }

    /**
     * Returns a new object with the given class name. The object instantiation
     * is done by calling a constructor with no parameters. If this constructor
     * does not exists or is not accessible or if any error occurs (e.g. class
     * not found) null is returned.
     * 
     * @param className
     *            String
     * @return Object
     */
    public static Object getNewObjWithEmptyConstructor(String className) {
        Class classToRun = getClass(className);
        if (classToRun == null) {
            logger.error("class '" + className + "' loading not successful  :-(");
            return null;
        }

        Constructor cons = null;
        try {
            cons = classToRun.getConstructor(null);
        } catch (SecurityException e1) {
            logger.error("error getting constructor without parameters for class '" + className + "'", e1);
        } catch (NoSuchMethodException e1) {
            logger.error("error getting constructor without parameters for class '" + className + "'", e1);
        }
        if (cons == null) {
            logger.fatal("Constructor not found. Your class '" + className + "' must implement an accessible constructor with no parameters");
            return null;
        }

        Object obj = null;
        try {
            obj = cons.newInstance(null);
        } catch (IllegalArgumentException e2) {
            logger.error("error instantiating class '" + className + "'", e2);
        } catch (InstantiationException e2) {
            logger.error("error instantiating class '" + className + "'", e2);
        } catch (IllegalAccessException e2) {
            logger.error("error instantiating class '" + className + "'", e2);
        } catch (InvocationTargetException e2) {
            logger.error("error instantiating class '" + className + "'", e2);
        }
        return obj;

    }

    public static IVisualisation createVisualisationInstance(String className, IEngineResults res) {

        Object obj = getNewObjWithEmptyConstructor(className);
        if (obj == null)
            return null;
        IVisualisation vis = null;
        try {
             vis = (IVisualisation) obj;
        } catch (ClassCastException e) {
           logger.fatal("class '"+className+"' must implement the interface "+IVisualisation.class.getName()+" or its subs"); 
           return null;
        }
        try {
            vis.init(res);
        } catch (IllegalArgumentException e2) {
            logger.error("class '"+className+"': error calling the init method with parameter IEngineResults", e2);
        }
        return vis;
    }

    /*
     * Beispiel fuer Constructor aufruf mit parametern public static Input
     * createInputInstance(String className, String dbName) { Input input =
     * null; try { Class classToRun = Class.forName(className); if (classToRun ==
     * null) { logger.fatal("class " + className + " not found"); } Class
     * dbString = String.class;
     * 
     * if (dbString == null) { logger.fatal("class " + className + " not
     * found"); }
     * 
     * Constructor cons = classToRun.getConstructor(new Class[] { dbString });
     * if (cons == null) { logger.fatal("Input Constructor not found"); } input =
     * (Input) cons.newInstance(new String[] { dbName }); } catch (Exception e) {
     *  }
     * 
     * return input; }
     */

    public static IInput createInputInstance(String className)  {
        Object obj = getNewObjWithEmptyConstructor(className);
        if (obj == null)
            return null;
        IInput input = null;
        try {
             input = (IInput) obj;
        } catch (ClassCastException e) {
           logger.fatal("class '"+className+"' must implement the interface "+IInput.class.getName()+" or its subs"); 
           return null;
        }
        return input;
    }

    
    public static IEngine createEngineInstance(String className) throws Exception {
        Object obj = getNewObjWithEmptyConstructor(className);
        if (obj == null)
            return null;
        IEngine engine = null;
        try {
             engine = (IEngine) obj;
        } catch (ClassCastException e) {
           logger.fatal("class '"+className+"' must implement the interface "+IEngine.class.getName()+" or its subs"); 
           return null;
        }
        try {
            //engine.init();
        } catch (IllegalArgumentException e2) {
            logger.error("class '"+className+"': error calling the init method", e2);
        }
        return engine;
    }
    
    
    public static IEngine1 createEngineInstance(String className, IInput setA) throws Exception {
        Object obj = getNewObjWithEmptyConstructor(className);
        if (obj == null)
            return null;
        IEngine1 engine = null;
        try {
             engine = (IEngine1) obj;
        } catch (ClassCastException e) {
           logger.fatal("class '"+className+"' must implement the interface "+IEngine1.class.getName()+" or its subs"); 
           return null;
        }
        try {
            engine.init(setA);
        } catch (IllegalArgumentException e2) {
            logger.error("class '"+className+"': error calling the init method with parameter IInput", e2);
        }
        return engine;
    }

    
    
    public static IEngine2 createEngineInstance(String className, IInput setA, IInput setB) throws Exception {
        /* HOT CLASS LOADER ???
         *         
         *         URLClassLoader cl;
        cl = new URLClassLoader(new URL[] { new File("c:\\\\").toURL() });
        logger.debug("nach");
        Class classToRun = cl.loadClass(className);
         * 
         */
        Object obj = getNewObjWithEmptyConstructor(className);
        if (obj == null)
            return null;
        IEngine2 engine = null;
        try {
             engine = (IEngine2) obj;
        } catch (ClassCastException e) {
           logger.fatal("class '"+className+"' must implement the interface "+IEngine2.class.getName()+" or its subs"); 
           return null;
        }
        try {
            engine.init(setA,setB);
        } catch (IllegalArgumentException e2) {
            logger.error("class '"+className+"': error calling the init method with parameter IInput x IInput", e2);
        }
        return engine;
    }

    public static ITest createTestInstance(String className) {
        Object obj = getNewObjWithEmptyConstructor(className);
        if (obj == null)
            return null;
        ITest test = null;
        try {
             test = (ITest) obj;
        } catch (Exception e) {
           logger.fatal("class '"+className+"' must implement the interface "+ITest.class.getName()+" or its subs"); 
           return null;
        }
        return test;
    }

    /*
    public static ITest createTestInstance(String className, IEngineResults res) {
        ITest test = null;
        try {
            Class classToRun = Class.forName(className);
            if (classToRun == null) {
                logger.fatal("class " + className + " not found :-(");
            }
            Constructor cons = classToRun.getConstructor(null);
            if (cons == null) {
                logger.fatal("Test() Constructor not found :-(");
            }
            test = (ITest) cons.newInstance(null);
            test.init(res);
        } catch (Exception e) {
            // logger.fatal(e.getStackTrace().toString());
          
        }
        return test;
    }

    public static ITest createTestInstance(String className, IEngineResults res, String str1) {
        ITest test = null;
        try {
            Class classToRun = Class.forName(className);
            if (classToRun == null) {
                logger.fatal("class " + className + " not found :-(");
            }
            Constructor cons = classToRun.getConstructor(null);
            if (cons == null) {
                logger.fatal("Test() Constructor not found :-(");
            }
            test = (Test) cons.newInstance(null);
            test.init(res, str1);
        } catch (Exception e) {
            // logger.fatal(e.getStackTrace().toString());
            
        }
        return test;
    }

    public static ITest createTestInstance(String className, IEngineResults res, String str1, String str2) {
        ITest test = null;
        try {
            Class classToRun = Class.forName(className);
            if (classToRun == null) {
                logger.fatal("class " + className + " not found :-(");
            }
            Constructor cons = classToRun.getConstructor(null);
            if (cons == null) {
                logger.fatal("Test() Constructor not found :-(");
            }
            test = (ITest) cons.newInstance(null);
            test.init(res, str1, str2);
        } catch (Exception e) {
            // logger.fatal(e.getStackTrace().toString());
            
        }
        return test;
    }
*/
}
